@@ -15,26 +15,26 @@
 import sys
 
 from keystoneclient.common import cms
-from oslo.utils import timeutils
+from oslo_config import cfg
+from oslo_log import log
+from oslo_serialization import jsonutils
+from oslo_utils import importutils
+from oslo_utils import timeutils
 import six
 
-from keystone.assignment import controllers as assignment_controllers
-from keystone.common import authorization
 from keystone.common import controller
 from keystone.common import dependency
 from keystone.common import wsgi
 from keystone import config
 from keystone.contrib import federation
 from keystone import exception
-from keystone.i18n import _, _LI
-from keystone.openstack.common import importutils
-from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log
+from keystone.i18n import _, _LI, _LW
+from keystone.resource import controllers as resource_controllers
 
 
 LOG = log.getLogger(__name__)
 
-CONF = config.CONF
+CONF = cfg.CONF
 
 # registry of authentication methods
 AUTH_METHODS = {}
@@ -125,7 +125,7 @@ class AuthContext(dict):
 # available for consumers. Consumers should probably not be getting
 # identity_api from this since it's available in global registry, then
 # identity_api should be removed from this list.
-@dependency.requires('assignment_api', 'identity_api', 'trust_api')
+@dependency.requires('identity_api', 'resource_api', 'trust_api')
 class AuthInfo(object):
     """Encapsulation of "auth" request."""
 
@@ -138,31 +138,31 @@ class AuthInfo(object):
     def __init__(self, context, auth=None):
         self.context = context
         self.auth = auth
-        self._scope_data = (None, None, None)
-        # self._scope_data is (domain_id, project_id, trust_ref)
-        # project scope: (None, project_id, None)
-        # domain scope: (domain_id, None, None)
-        # trust scope: (None, None, trust_ref)
-        # unscoped: (None, None, None)
+        self._scope_data = (None, None, None, None)
+        # self._scope_data is (domain_id, project_id, trust_ref, unscoped)
+        # project scope: (None, project_id, None, None)
+        # domain scope: (domain_id, None, None, None)
+        # trust scope: (None, None, trust_ref, None)
+        # unscoped: (None, None, None, 'unscoped')
 
     def _assert_project_is_enabled(self, project_ref):
         # ensure the project is enabled
         try:
-            self.assignment_api.assert_project_enabled(
+            self.resource_api.assert_project_enabled(
                 project_id=project_ref['id'],
                 project=project_ref)
         except AssertionError as e:
-            LOG.warning(e)
+            LOG.warning(six.text_type(e))
             six.reraise(exception.Unauthorized, exception.Unauthorized(e),
                         sys.exc_info()[2])
 
     def _assert_domain_is_enabled(self, domain_ref):
         try:
-            self.assignment_api.assert_domain_enabled(
+            self.resource_api.assert_domain_enabled(
                 domain_id=domain_ref['id'],
                 domain=domain_ref)
         except AssertionError as e:
-            LOG.warning(e)
+            LOG.warning(six.text_type(e))
             six.reraise(exception.Unauthorized, exception.Unauthorized(e),
                         sys.exc_info()[2])
 
@@ -175,12 +175,12 @@ class AuthInfo(object):
                                             target='domain')
         try:
             if domain_name:
-                domain_ref = self.assignment_api.get_domain_by_name(
+                domain_ref = self.resource_api.get_domain_by_name(
                     domain_name)
             else:
-                domain_ref = self.assignment_api.get_domain(domain_id)
+                domain_ref = self.resource_api.get_domain(domain_id)
         except exception.DomainNotFound as e:
-            LOG.exception(e)
+            LOG.exception(six.text_type(e))
             raise exception.Unauthorized(e)
         self._assert_domain_is_enabled(domain_ref)
         return domain_ref
@@ -198,16 +198,15 @@ class AuthInfo(object):
                     raise exception.ValidationError(attribute='domain',
                                                     target='project')
                 domain_ref = self._lookup_domain(project_info['domain'])
-                project_ref = self.assignment_api.get_project_by_name(
+                project_ref = self.resource_api.get_project_by_name(
                     project_name, domain_ref['id'])
             else:
-                project_ref = self.assignment_api.get_project(project_id)
+                project_ref = self.resource_api.get_project(project_id)
                 # NOTE(morganfainberg): The _lookup_domain method will raise
                 # exception.Unauthorized if the domain isn't found or is
                 # disabled.
                 self._lookup_domain({'id': project_ref['domain_id']})
         except exception.ProjectNotFound as e:
-            LOG.exception(e)
             raise exception.Unauthorized(e)
         self._assert_project_is_enabled(project_ref)
         return project_ref
@@ -228,17 +227,20 @@ class AuthInfo(object):
             return
         if sum(['project' in self.auth['scope'],
                 'domain' in self.auth['scope'],
+                'unscoped' in self.auth['scope'],
                 'OS-TRUST:trust' in self.auth['scope']]) != 1:
             raise exception.ValidationError(
-                attribute='project, domain, or OS-TRUST:trust',
+                attribute='project, domain, OS-TRUST:trust or unscoped',
                 target='scope')
-
+        if 'unscoped' in self.auth['scope']:
+            self._scope_data = (None, None, None, 'unscoped')
+            return
         if 'project' in self.auth['scope']:
             project_ref = self._lookup_project(self.auth['scope']['project'])
-            self._scope_data = (None, project_ref['id'], None)
+            self._scope_data = (None, project_ref['id'], None, None)
         elif 'domain' in self.auth['scope']:
             domain_ref = self._lookup_domain(self.auth['scope']['domain'])
-            self._scope_data = (domain_ref['id'], None, None)
+            self._scope_data = (domain_ref['id'], None, None, None)
         elif 'OS-TRUST:trust' in self.auth['scope']:
             if not CONF.trust.enabled:
                 raise exception.Forbidden('Trusts are disabled.')
@@ -248,9 +250,9 @@ class AuthInfo(object):
             if trust_ref.get('project_id') is not None:
                 project_ref = self._lookup_project(
                     {'id': trust_ref['project_id']})
-                self._scope_data = (None, project_ref['id'], trust_ref)
+                self._scope_data = (None, project_ref['id'], trust_ref, None)
             else:
-                self._scope_data = (None, None, trust_ref)
+                self._scope_data = (None, None, trust_ref, None)
 
     def _validate_auth_methods(self):
         if 'identity' not in self.auth:
@@ -313,20 +315,22 @@ class AuthInfo(object):
 
         Verify and return the scoping information.
 
-        :returns: (domain_id, project_id, trust_ref).
-                   If scope to a project, (None, project_id, None)
+        :returns: (domain_id, project_id, trust_ref, unscoped).
+                   If scope to a project, (None, project_id, None, None)
                    will be returned.
-                   If scoped to a domain, (domain_id, None, None)
+                   If scoped to a domain, (domain_id, None, None, None)
                    will be returned.
-                   If scoped to a trust, (None, project_id, trust_ref),
+                   If scoped to a trust, (None, project_id, trust_ref, None),
                    Will be returned, where the project_id comes from the
                    trust definition.
-                   If unscoped, (None, None, None) will be returned.
+                   If unscoped, (None, None, None, 'unscoped') will be
+                   returned.
 
         """
         return self._scope_data
 
-    def set_scope(self, domain_id=None, project_id=None, trust=None):
+    def set_scope(self, domain_id=None, project_id=None, trust=None,
+                  unscoped=None):
         """Set scope information."""
         if domain_id and project_id:
             msg = _('Scoping to both domain and project is not allowed')
@@ -337,11 +341,11 @@ class AuthInfo(object):
         if project_id and trust:
             msg = _('Scoping to both project and trust is not allowed')
             raise ValueError(msg)
-        self._scope_data = (domain_id, project_id, trust)
+        self._scope_data = (domain_id, project_id, trust, unscoped)
 
 
 @dependency.requires('assignment_api', 'catalog_api', 'identity_api',
-                     'token_provider_api', 'trust_api')
+                     'resource_api', 'token_provider_api', 'trust_api')
 class Auth(controller.V3Controller):
 
     # Note(atiwari): From V3 auth controller code we are
@@ -374,7 +378,7 @@ class Auth(controller.V3Controller):
             if auth_context.get('access_token_id'):
                 auth_info.set_scope(None, auth_context['project_id'], None)
             self._check_and_set_default_scoping(auth_info, auth_context)
-            (domain_id, project_id, trust) = auth_info.get_scope()
+            (domain_id, project_id, trust, unscoped) = auth_info.get_scope()
 
             method_names = auth_info.get_method_names()
             method_names += auth_context.get('method_names', [])
@@ -403,7 +407,7 @@ class Auth(controller.V3Controller):
             raise exception.Unauthorized(e)
 
     def _check_and_set_default_scoping(self, auth_info, auth_context):
-        (domain_id, project_id, trust) = auth_info.get_scope()
+        (domain_id, project_id, trust, unscoped) = auth_info.get_scope()
         if trust:
             project_id = trust['project_id']
         if domain_id or project_id or trust:
@@ -414,11 +418,15 @@ class Auth(controller.V3Controller):
         if federation.IDENTITY_PROVIDER in auth_context:
             return
 
+        # Do not scope if request is for explicitly unscoped token
+        if unscoped is not None:
+            return
+
         # fill in default_project_id if it is available
         try:
             user_ref = self.identity_api.get_user(auth_context['user_id'])
         except exception.UserNotFound as e:
-            LOG.exception(e)
+            LOG.exception(six.text_type(e))
             raise exception.Unauthorized(e)
 
         default_project_id = user_ref.get('default_project_id')
@@ -428,9 +436,9 @@ class Auth(controller.V3Controller):
 
         # make sure user's default project is legit before scoping to it
         try:
-            default_project_ref = self.assignment_api.get_project(
+            default_project_ref = self.resource_api.get_project(
                 default_project_id)
-            default_project_domain_ref = self.assignment_api.get_domain(
+            default_project_domain_ref = self.resource_api.get_domain(
                 default_project_ref['domain_id'])
             if (default_project_ref.get('enabled', True) and
                     default_project_domain_ref.get('enabled', True)):
@@ -438,25 +446,26 @@ class Auth(controller.V3Controller):
                         user_ref['id'], default_project_id):
                     auth_info.set_scope(project_id=default_project_id)
                 else:
-                    msg = _("User %(user_id)s doesn't have access to"
-                            " default project %(project_id)s. The token will"
-                            " be unscoped rather than scoped to the project.")
+                    msg = _LW("User %(user_id)s doesn't have access to"
+                              " default project %(project_id)s. The token"
+                              " will be unscoped rather than scoped to the"
+                              " project.")
                     LOG.warning(msg,
                                 {'user_id': user_ref['id'],
                                  'project_id': default_project_id})
             else:
-                msg = _("User %(user_id)s's default project %(project_id)s is"
-                        " disabled. The token will be unscoped rather than"
-                        " scoped to the project.")
+                msg = _LW("User %(user_id)s's default project %(project_id)s"
+                          " is disabled. The token will be unscoped rather"
+                          " than scoped to the project.")
                 LOG.warning(msg,
                             {'user_id': user_ref['id'],
                              'project_id': default_project_id})
         except (exception.ProjectNotFound, exception.DomainNotFound):
             # default project or default project domain doesn't exist,
             # will issue unscoped token instead
-            msg = _("User %(user_id)s's default project %(project_id)s not"
-                    " found. The token will be unscoped rather than"
-                    " scoped to the project.")
+            msg = _LW("User %(user_id)s's default project %(project_id)s not"
+                      " found. The token will be unscoped rather than"
+                      " scoped to the project.")
             LOG.warning(msg, {'user_id': user_ref['id'],
                               'project_id': default_project_id})
 
@@ -477,11 +486,11 @@ class Auth(controller.V3Controller):
                 # In those cases, it is correct to not register an
                 # 'external' plugin;  if there is both an 'external' and a
                 # 'kerberos' plugin, it would run the check on identity twice.
-                pass
+                LOG.debug("No 'external' plugin is registered.")
             except exception.Unauthorized:
                 # If external fails then continue and attempt to determine
                 # user identity using remaining auth methods
-                pass
+                LOG.debug("Authorization failed for 'external' auth method.")
 
         # need to aggregate the results in case two or more methods
         # are specified
@@ -537,7 +546,7 @@ class Auth(controller.V3Controller):
         for t in tokens:
             expires = t['expires']
             if not (expires and isinstance(expires, six.text_type)):
-                    t['expires'] = timeutils.isotime(expires)
+                t['expires'] = timeutils.isotime(expires)
         data = {'revoked': tokens}
         json_data = jsonutils.dumps(data)
         signed_text = cms.cms_sign_text(json_data,
@@ -546,17 +555,11 @@ class Auth(controller.V3Controller):
 
         return {'signed': signed_text}
 
-    def get_auth_context(self, context):
-        # TODO(dolphm): this method of accessing the auth context is terrible,
-        # but context needs to be refactored to always have reasonable values.
-        env_context = context.get('environment', {})
-        return env_context.get(authorization.AUTH_CONTEXT_ENV, {})
-
     def _combine_lists_uniquely(self, a, b):
         # it's most likely that only one of these will be filled so avoid
         # the combination if possible.
         if a and b:
-            return dict((x['id'], x) for x in a + b).values()
+            return {x['id']: x for x in a + b}.values()
         else:
             return a or b
 
@@ -579,7 +582,7 @@ class Auth(controller.V3Controller):
             grp_refs = self.assignment_api.list_projects_for_groups(group_ids)
 
         refs = self._combine_lists_uniquely(user_refs, grp_refs)
-        return assignment_controllers.ProjectV3.wrap_collection(context, refs)
+        return resource_controllers.ProjectV3.wrap_collection(context, refs)
 
     @controller.protected()
     def get_auth_domains(self, context):
@@ -600,7 +603,7 @@ class Auth(controller.V3Controller):
             grp_refs = self.assignment_api.list_domains_for_groups(group_ids)
 
         refs = self._combine_lists_uniquely(user_refs, grp_refs)
-        return assignment_controllers.DomainV3.wrap_collection(context, refs)
+        return resource_controllers.DomainV3.wrap_collection(context, refs)
 
     @controller.protected()
     def get_auth_catalog(self, context):
