@@ -17,13 +17,13 @@ import datetime
 
 from dogpile.cache import api
 from dogpile.cache import util as dp_util
-from oslo.utils import timeutils
+from oslo_log import log
+from oslo_utils import importutils
+from oslo_utils import timeutils
 import six
 
 from keystone import exception
-from keystone.i18n import _
-from keystone.openstack.common import importutils
-from keystone.openstack.common import log
+from keystone.i18n import _, _LW
 
 
 NO_VALUE = api.NO_VALUE
@@ -404,10 +404,10 @@ class MongoApi(object):
                 existing_value = index_data['expireAfterSeconds']
                 fld_present = 'doc_date' in index_data['key'][0]
                 if fld_present and existing_value > -1 and ttl_seconds < 1:
-                    msg = _('TTL index already exists on db collection '
-                            '<%(c_name)s>, remove index <%(indx_name)s> first'
-                            ' to make updated mongo_ttl_seconds value to be '
-                            ' effective')
+                    msg = _LW('TTL index already exists on db collection '
+                              '<%(c_name)s>, remove index <%(indx_name)s> '
+                              'first to make updated mongo_ttl_seconds value '
+                              'to be  effective')
                     LOG.warn(msg, {'c_name': coll_name,
                                    'indx_name': indx_name})
 
@@ -422,14 +422,13 @@ class MongoApi(object):
 
     def get_multi(self, keys):
         db_results = self._get_results_as_dict(keys)
-        return dict((doc['_id'], doc['value']) for doc in
-                    six.itervalues(db_results))
+        return {doc['_id']: doc['value'] for doc in six.itervalues(db_results)}
 
     def _get_results_as_dict(self, keys):
         critieria = {'_id': {'$in': keys}}
         db_results = self.get_cache_collection().find(spec=critieria,
                                                       **self.meth_kwargs)
-        return dict((doc['_id'], doc) for doc in db_results)
+        return {doc['_id']: doc for doc in db_results}
 
     def set(self, key, value):
         doc_date = self._get_doc_date()
