@@ -25,21 +25,27 @@ import sys
 import eventlet
 import eventlet.wsgi
 import greenlet
+from oslo_config import cfg
+from oslo_log import log
+from oslo_log import loggers
 
-from keystone.common import config
-from keystone.i18n import _
-from keystone.i18n import _LE
-from keystone.i18n import _LI
-from keystone.openstack.common import log
+from keystone.i18n import _LE, _LI
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 
 
 LOG = log.getLogger(__name__)
 
+# The size of a pool that is used to spawn a single green thread in which
+# a wsgi server is then started. The size of one is enough, because in case
+# of several workers the parent process forks and each child gets a copy
+# of a pool, which does not include any greenthread object as the spawn is
+# done after the fork.
+POOL_SIZE = 1
 
-class EventletFilteringLogger(log.WritableLogger):
+
+class EventletFilteringLogger(loggers.WritableLogger):
     # NOTE(morganfainberg): This logger is designed to filter out specific
     # Tracebacks to limit the amount of data that eventlet can log. In the
     # case of broken sockets (EPIPE and ECONNRESET), we are seeing a huge
@@ -63,12 +69,13 @@ class EventletFilteringLogger(log.WritableLogger):
 class Server(object):
     """Server class to manage multiple WSGI sockets and applications."""
 
-    def __init__(self, application, host=None, port=None, threads=1000,
-                 keepalive=False, keepidle=None):
+    def __init__(self, application, host=None, port=None, keepalive=False,
+                 keepidle=None):
         self.application = application
         self.host = host or '0.0.0.0'
         self.port = port or 0
-        self.pool = eventlet.GreenPool(threads)
+        # Pool for a green thread in which wsgi server will be running
+        self.pool = eventlet.GreenPool(POOL_SIZE)
         self.socket_info = {}
         self.greenthread = None
         self.do_ssl = False
@@ -136,8 +143,7 @@ class Server(object):
         if self.keepalive:
             dup_socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
 
-            # This option isn't available in the OS X version of eventlet
-            if hasattr(socket, 'TCP_KEEPIDLE') and self.keepidle is not None:
+            if self.keepidle is not None:
                 dup_socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE,
                                       self.keepidle)
 
@@ -178,17 +184,17 @@ class Server(object):
         pass
 
     def _run(self, application, socket):
-        """Start a WSGI server in a new green thread."""
+        """Start a WSGI server with a new green thread pool."""
         logger = log.getLogger('eventlet.wsgi.server')
-        socket_timeout = CONF.client_socket_timeout or None
+        socket_timeout = CONF.eventlet_server.client_socket_timeout or None
         try:
             eventlet.wsgi.server(
                 socket, application, log=EventletFilteringLogger(logger),
-                debug=False, keepalive=CONF.wsgi_keep_alive,
+                debug=False, keepalive=CONF.eventlet_server.wsgi_keep_alive,
                 socket_timeout=socket_timeout)
         except greenlet.GreenletExit:
             # Wait until all servers have completed running
             pass
         except Exception:
-            LOG.exception(_('Server error'))
+            LOG.exception(_LE('Server error'))
             raise
