@@ -16,24 +16,24 @@
 Keystone Memcached dogpile.cache backend implementation.
 """
 
-import random
+import random as _random
 import time
 
 from dogpile.cache import api
 from dogpile.cache.backends import memcached
+from oslo_config import cfg
+from oslo_log import log
 
 from keystone.common.cache.backends import memcache_pool
 from keystone.common import manager
-from keystone import config
 from keystone import exception
 from keystone.i18n import _
-from keystone.openstack.common import log
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 NO_VALUE = api.NO_VALUE
-
+random = _random.SystemRandom()
 
 VALID_DOGPILE_BACKENDS = dict(
     pylibmc=memcached.PylibmcBackend,
@@ -142,22 +142,21 @@ class MemcachedBackend(manager.Manager):
             # all ``set`` and ``set_multi`` calls by the driver, by calling
             # the client directly it is possible to exclude the ``time``
             # argument to the memcached server.
-            new_mapping = dict((k, mapping[k]) for k in no_expiry_keys)
+            new_mapping = {k: mapping[k] for k in no_expiry_keys}
             set_arguments = self._get_set_arguments_driver_attr(
                 exclude_expiry=True)
             self.driver.client.set_multi(new_mapping, **set_arguments)
 
         if has_expiry_keys:
-            new_mapping = dict((k, mapping[k]) for k in has_expiry_keys)
+            new_mapping = {k: mapping[k] for k in has_expiry_keys}
             self.driver.set_multi(new_mapping)
 
     @classmethod
     def from_config_dict(cls, config_dict, prefix):
         prefix_len = len(prefix)
         return cls(
-            dict((key[prefix_len:], config_dict[key])
-                 for key in config_dict
-                 if key.startswith(prefix)))
+            {key[prefix_len:]: config_dict[key] for key in config_dict
+             if key.startswith(prefix)})
 
     @property
     def key_mangler(self):
