@@ -19,15 +19,39 @@ import sys
 
 import migrate
 from migrate import exceptions
+from oslo_config import cfg
+from oslo_db.sqlalchemy import migration
+from oslo_serialization import jsonutils
+from oslo_utils import importutils
 import six
 import sqlalchemy
 
 from keystone.common import sql
+from keystone.common.sql import migrate_repo
 from keystone import contrib
 from keystone import exception
-from keystone.openstack.common.db.sqlalchemy import migration
-from keystone.openstack.common.gettextutils import _
-from keystone.openstack.common import importutils
+from keystone.i18n import _
+
+
+CONF = cfg.CONF
+DEFAULT_EXTENSIONS = ['endpoint_filter',
+                      'endpoint_policy',
+                      'federation',
+                      'oauth1',
+                      'revoke',
+                      ]
+
+
+def get_default_domain():
+    # Return the reference used for the default domain structure during
+    # sql migrations.
+    return {
+        'id': CONF.identity.default_domain_id,
+        'name': 'Default',
+        'enabled': True,
+        'extra': jsonutils.dumps({'description': 'Owns users and tenants '
+                                                 '(i.e. projects) available '
+                                                 'on Identity API v2.'})}
 
 
 #  Different RDBMSs use different schemes for naming the Foreign Key
@@ -36,14 +60,14 @@ from keystone.openstack.common import importutils
 #  This fails on MySQL.
 def get_constraints_names(table, column_name):
     fkeys = [fk.name for fk in table.constraints
-             if (column_name in fk.columns and
-                 isinstance(fk, sqlalchemy.ForeignKeyConstraint))]
+             if (isinstance(fk, sqlalchemy.ForeignKeyConstraint) and
+                 column_name in fk.columns)]
     return fkeys
 
 
 #  remove_constraints and add_constraints both accept a list of dictionaries
 #  that contain:
-#  {'table': a sqlalchemy table. The constraint is added to to dropped from
+#  {'table': a sqlalchemy table. The constraint is added to dropped from
 #           this table.
 #  'fk_column': the name of a column on the above table,  The constraint
 #               is added to or dropped from this column
@@ -115,69 +139,120 @@ def find_migrate_repo(package=None, repo_name='migrate_repo'):
     raise exception.MigrationNotProvided(package.__name__, path)
 
 
-def _fix_migration_37(engine):
-    """Fix the region table to be InnoDB and Charset UTF8.
+def _sync_common_repo(version):
+    abs_path = find_migrate_repo()
+    init_version = migrate_repo.DB_INIT_VERSION
+    engine = sql.get_engine()
+    _assert_not_schema_downgrade(version=version)
+    migration.db_sync(engine, abs_path, version=version,
+                      init_version=init_version)
 
-    This function is to work around bug #1334779. This has occurred because
-    the original migration 37 did not specify InnoDB and charset utf8. Due
+
+def _fix_federation_tables(engine):
+    """Fix the identity_provider, federation_protocol and mapping tables
+     to be InnoDB and Charset UTF8.
+
+    This function is to work around bug #1426334. This has occurred because
+    the original migration did not specify InnoDB and charset utf8. Due
     to the sanity_check, a deployer can get wedged here and require manual
     database changes to fix.
     """
-    # NOTE(morganfainberg): Extra defensive here, check to make sure we really
-    # are mysql before trying to perform the alters.
+    # NOTE(marco-fargetta) This is a workaround to "fix" that tables only
+    # if we're under MySQL
     if engine.name == 'mysql':
-        # * Make sure the engine is InnoDB
-        engine.execute('ALTER TABLE region Engine=InnoDB')
-        # * Make sure the character set is utf8
-        engine.execute('ALTER TABLE region CONVERT TO CHARACTER SET utf8')
+        # * Disable any check for the foreign keys because they prevent the
+        # alter table to execute
+        engine.execute("SET foreign_key_checks = 0")
+        # * Make the tables using InnoDB engine
+        engine.execute("ALTER TABLE identity_provider Engine=InnoDB")
+        engine.execute("ALTER TABLE federation_protocol Engine=InnoDB")
+        engine.execute("ALTER TABLE mapping Engine=InnoDB")
+        # * Make the tables using utf8 encoding
+        engine.execute("ALTER TABLE identity_provider "
+                       "CONVERT TO CHARACTER SET utf8")
+        engine.execute("ALTER TABLE federation_protocol "
+                       "CONVERT TO CHARACTER SET utf8")
+        engine.execute("ALTER TABLE mapping CONVERT TO CHARACTER SET utf8")
+        # * Revert the foreign keys check back
+        engine.execute("SET foreign_key_checks = 1")
 
 
-def sync_database_to_version(extension=None, version=None):
-    if not extension:
-        abs_path = find_migrate_repo()
-    else:
+def _assert_not_schema_downgrade(extension=None, version=None):
+    if version is not None:
         try:
-            package_name = '.'.join((contrib.__name__, extension))
-            package = importutils.import_module(package_name)
-        except ImportError:
-            raise ImportError(_("%s extension does not exist.")
-                              % package_name)
-        try:
-            abs_path = find_migrate_repo(package)
-            try:
-                migration.db_version_control(sql.get_engine(), abs_path)
-            # Register the repo with the version control API
-            # If it already knows about the repo, it will throw
-            # an exception that we can safely ignore
-            except exceptions.DatabaseAlreadyControlledError:
-                pass
-        except exception.MigrationNotProvided as e:
-            print(e)
-            sys.exit(1)
+            current_ver = int(six.text_type(get_db_version(extension)))
+            if int(version) < current_ver:
+                raise migration.exception.DbMigrationError()
+        except exceptions.DatabaseNotControlledError:
+            # NOTE(morganfainberg): The database is not controlled, this action
+            # cannot be a downgrade.
+            pass
 
+
+def _sync_extension_repo(extension, version):
+    init_version = 0
     engine = sql.get_engine()
+
     try:
-        migration.db_sync(engine, abs_path, version=version)
+        package_name = '.'.join((contrib.__name__, extension))
+        package = importutils.import_module(package_name)
+    except ImportError:
+        raise ImportError(_("%s extension does not exist.")
+                          % package_name)
+    try:
+        abs_path = find_migrate_repo(package)
+        try:
+            migration.db_version_control(sql.get_engine(), abs_path)
+        # Register the repo with the version control API
+        # If it already knows about the repo, it will throw
+        # an exception that we can safely ignore
+        except exceptions.DatabaseAlreadyControlledError:
+            pass
+    except exception.MigrationNotProvided as e:
+        print(e)
+        sys.exit(1)
+
+    _assert_not_schema_downgrade(extension=extension, version=version)
+
+    try:
+        migration.db_sync(engine, abs_path, version=version,
+                          init_version=init_version)
     except ValueError:
-        # NOTE(morganfainberg): ValueError is raised from the sanity check (
-        # verifies that tables are utf8 under mysql). The region table was not
-        # initially built with InnoDB and utf8 as part of the table arguments
-        # when the migration was initially created. Bug #1334779 is a scenario
-        # where the deployer can get wedged, unable to upgrade or downgrade.
-        # This is a workaround to "fix" that table if we're under MySQL.
-        if (not extension and engine.name == 'mysql' and
-                six.text_type(get_db_version()) == '37'):
-            _fix_migration_37(engine)
-            # Try the migration a second time now that we've done the
-            # un-wedge work.
-            migration.db_sync(engine, abs_path, version=version)
+        # NOTE(marco-fargetta): ValueError is raised from the sanity check (
+        # verifies that tables are utf8 under mysql). The federation_protocol,
+        # identity_provider and mapping tables were not initially built with
+        # InnoDB and utf8 as part of the table arguments when the migration
+        # was initially created. Bug #1426334 is a scenario where the deployer
+        # can get wedged, unable to upgrade or downgrade.
+        # This is a workaround to "fix" those tables if we're under MySQL and
+        # the version is before the 6 because before the tables were introduced
+        # before and patched when migration 5 was available
+        if engine.name == 'mysql' and \
+           int(six.text_type(get_db_version(extension))) < 6:
+            _fix_federation_tables(engine)
+            # The migration is applied again after the fix
+            migration.db_sync(engine, abs_path, version=version,
+                              init_version=init_version)
         else:
             raise
 
 
+def sync_database_to_version(extension=None, version=None):
+    if not extension:
+        _sync_common_repo(version)
+        # If version is greater than 0, it is for the common
+        # repository only, and only that will be synchronized.
+        if version is None:
+            for default_extension in DEFAULT_EXTENSIONS:
+                _sync_extension_repo(default_extension, version)
+    else:
+        _sync_extension_repo(extension, version)
+
+
 def get_db_version(extension=None):
     if not extension:
-        return migration.db_version(sql.get_engine(), find_migrate_repo(), 0)
+        return migration.db_version(sql.get_engine(), find_migrate_repo(),
+                                    migrate_repo.DB_INIT_VERSION)
 
     try:
         package_name = '.'.join((contrib.__name__, extension))
