@@ -19,27 +19,31 @@
 """Utility methods for working with WSGI servers."""
 
 import copy
+import itertools
+import wsgiref.util
 
-from oslo import i18n
+from oslo_config import cfg
+import oslo_i18n
+from oslo_log import log
+from oslo_serialization import jsonutils
+from oslo_utils import importutils
+from oslo_utils import strutils
 import routes.middleware
 import six
 import webob.dec
 import webob.exc
 
-from keystone.common import config
 from keystone.common import dependency
+from keystone.common import json_home
 from keystone.common import utils
 from keystone import exception
 from keystone.i18n import _
 from keystone.i18n import _LI
 from keystone.i18n import _LW
 from keystone.models import token_model
-from keystone.openstack.common import importutils
-from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 # Environment variable used to pass the request context
@@ -113,7 +117,7 @@ def best_match_language(req):
     if not req.accept_language:
         return None
     return req.accept_language.best_match(
-        i18n.get_available_languages('keystone'))
+        oslo_i18n.get_available_languages('keystone'))
 
 
 class BaseApplication(object):
@@ -188,13 +192,22 @@ class Application(BaseApplication):
         arg_dict = req.environ['wsgiorg.routing_args'][1]
         action = arg_dict.pop('action')
         del arg_dict['controller']
-        LOG.debug('arg_dict: %s', arg_dict)
 
         # allow middleware up the stack to provide context, params and headers.
         context = req.environ.get(CONTEXT_ENV, {})
         context['query_string'] = dict(six.iteritems(req.params))
         context['headers'] = dict(six.iteritems(req.headers))
         context['path'] = req.environ['PATH_INFO']
+        scheme = (None if not CONF.secure_proxy_ssl_header
+                  else req.environ.get(CONF.secure_proxy_ssl_header))
+        if scheme:
+            # NOTE(andrey-mp): "wsgi.url_scheme" contains the protocol used
+            # before the proxy removed it ('https' usually). So if
+            # the webob.Request instance is modified in order to use this
+            # scheme instead of the one defined by API, the call to
+            # webob.Request.relative_url() will return a URL with the correct
+            # scheme.
+            req.environ['wsgi.url_scheme'] = scheme
         context['host_url'] = req.host_url
         params = req.environ.get(PARAMS_ENV, {})
         # authentication and authorization attributes are set as environment
@@ -214,9 +227,11 @@ class Application(BaseApplication):
         # NOTE(morganfainberg): use the request method to normalize the
         # response code between GET and HEAD requests. The HTTP status should
         # be the same.
-        req_method = req.environ['REQUEST_METHOD'].upper()
+        LOG.info('%(req_method)s %(uri)s', {
+            'req_method': req.environ['REQUEST_METHOD'].upper(),
+            'uri': wsgiref.util.request_uri(req.environ),
+        })
 
-        # NOTE(vish): make sure we have no unicode keys for py2.6.
         params = self._normalize_dict(params)
 
         try:
@@ -229,16 +244,16 @@ class Application(BaseApplication):
             return render_exception(e, context=context,
                                     user_locale=best_match_language(req))
         except exception.Error as e:
-            LOG.warning(e)
+            LOG.warning(six.text_type(e))
             return render_exception(e, context=context,
                                     user_locale=best_match_language(req))
         except TypeError as e:
-            LOG.exception(e)
+            LOG.exception(six.text_type(e))
             return render_exception(exception.ValidationError(e),
                                     context=context,
                                     user_locale=best_match_language(req))
         except Exception as e:
-            LOG.exception(e)
+            LOG.exception(six.text_type(e))
             return render_exception(exception.UnexpectedError(exception=e),
                                     context=context,
                                     user_locale=best_match_language(req))
@@ -254,7 +269,7 @@ class Application(BaseApplication):
 
         response_code = self._get_response_code(req)
         return render_response(body=result, status=response_code,
-                               method=req_method)
+                               method=req.environ['REQUEST_METHOD'])
 
     def _get_response_code(self, req):
         req_method = req.environ['REQUEST_METHOD']
@@ -265,11 +280,10 @@ class Application(BaseApplication):
         return code
 
     def _normalize_arg(self, arg):
-        return str(arg).replace(':', '_').replace('-', '_')
+        return arg.replace(':', '_').replace('-', '_')
 
     def _normalize_dict(self, d):
-        return dict([(self._normalize_arg(k), v)
-                     for (k, v) in six.iteritems(d)])
+        return {self._normalize_arg(k): v for (k, v) in six.iteritems(d)}
 
     def assert_admin(self, context):
         if not context['is_admin']:
@@ -355,7 +369,11 @@ class Application(BaseApplication):
         url = CONF['%s_endpoint' % endpoint_type]
 
         if url:
-            url = url % CONF
+            substitutions = dict(
+                itertools.chain(six.iteritems(CONF),
+                                six.iteritems(CONF.eventlet_server)))
+
+            url = url % substitutions
         else:
             # NOTE(jamielennox): if url is not set via the config file we
             # should set it relative to the url that the user used to get here
@@ -433,16 +451,16 @@ class Middleware(Application):
             response = request.get_response(self.application)
             return self.process_response(request, response)
         except exception.Error as e:
-            LOG.warning(e)
+            LOG.warning(six.text_type(e))
             return render_exception(e, request=request,
                                     user_locale=best_match_language(request))
         except TypeError as e:
-            LOG.exception(e)
+            LOG.exception(six.text_type(e))
             return render_exception(exception.ValidationError(e),
                                     request=request,
                                     user_locale=best_match_language(request))
         except Exception as e:
-            LOG.exception(e)
+            LOG.exception(six.text_type(e))
             return render_exception(exception.UnexpectedError(exception=e),
                                     request=request,
                                     user_locale=best_match_language(request))
@@ -462,11 +480,11 @@ class Debug(Middleware):
             LOG.debug('%s %s %s', ('*' * 20), 'REQUEST ENVIRON', ('*' * 20))
             for key, value in req.environ.items():
                 LOG.debug('%s = %s', key,
-                          log.mask_password(value))
+                          strutils.mask_password(value))
             LOG.debug('')
             LOG.debug('%s %s %s', ('*' * 20), 'REQUEST BODY', ('*' * 20))
             for line in req.body_file:
-                LOG.debug('%s', log.mask_password(line))
+                LOG.debug('%s', strutils.mask_password(line))
             LOG.debug('')
 
         resp = req.get_response(self.application)
@@ -584,7 +602,7 @@ class ExtensionRouter(Router):
             mapper = routes.Mapper()
         self.application = application
         self.add_routes(mapper)
-        mapper.connect('{path_info:.*}', controller=self.application)
+        mapper.connect('/{path_info:.*}', controller=self.application)
         super(ExtensionRouter, self).__init__(mapper)
 
     def add_routes(self, mapper):
@@ -638,7 +656,7 @@ class RoutersBase(object):
                       get_action=None, head_action=None, get_head_action=None,
                       put_action=None, post_action=None, patch_action=None,
                       delete_action=None, get_post_action=None,
-                      path_vars=None):
+                      path_vars=None, status=json_home.Status.STABLE):
         if get_head_action:
             getattr(controller, get_head_action)  # ensure the attribute exists
             mapper.connect(path, controller=controller, action=get_head_action,
@@ -680,6 +698,8 @@ class RoutersBase(object):
         else:
             resource_data['href'] = path
 
+        json_home.Status.update_resource_data(resource_data, status)
+
         self.v3_resources.append((rel, resource_data))
 
 
@@ -747,7 +767,7 @@ def render_response(body=None, status=None, headers=None, method=None):
                           status='%s %s' % status,
                           headerlist=headers)
 
-    if method == 'HEAD':
+    if method and method.upper() == 'HEAD':
         # NOTE(morganfainberg): HEAD requests should return the same status
         # as a GET request and same headers (including content-type and
         # content-length). The webob.Response object automatically changes
@@ -768,7 +788,7 @@ def render_exception(error, context=None, request=None, user_locale=None):
     """Forms a WSGI response based on the current error."""
 
     error_message = error.args[0]
-    message = i18n.translate(error_message, desired_locale=user_locale)
+    message = oslo_i18n.translate(error_message, desired_locale=user_locale)
     if message is error_message:
         # translate() didn't do anything because it wasn't a Message,
         # convert to a string.
@@ -790,9 +810,12 @@ def render_exception(error, context=None, request=None, user_locale=None):
             if context:
                 url = Application.base_url(context, 'public')
             else:
-                url = 'http://localhost:%d' % CONF.public_port
+                url = 'http://localhost:%d' % CONF.eventlet_server.public_port
         else:
-            url = url % CONF
+            substitutions = dict(
+                itertools.chain(six.iteritems(CONF),
+                                six.iteritems(CONF.eventlet_server)))
+            url = url % substitutions
 
         headers.append(('WWW-Authenticate', 'Keystone uri="%s"' % url))
     return render_response(status=(error.code, error.title),
