@@ -13,23 +13,23 @@
 import abc
 import datetime
 
-from oslo.utils import timeutils
+from oslo_config import cfg
+from oslo_log import log
+from oslo_utils import timeutils
 import six
 
 from keystone.common import cache
 from keystone.common import dependency
 from keystone.common import extension
 from keystone.common import manager
-from keystone import config
 from keystone.contrib.revoke import model
 from keystone import exception
 from keystone.i18n import _
 from keystone import notifications
-from keystone.openstack.common import log
 from keystone.openstack.common import versionutils
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -52,9 +52,7 @@ EXTENSION_DATA = {
 extension.register_admin_extension(EXTENSION_DATA['alias'], EXTENSION_DATA)
 extension.register_public_extension(EXTENSION_DATA['alias'], EXTENSION_DATA)
 
-SHOULD_CACHE = cache.should_cache_fn('revoke')
-# TODO(ayoung): migrate from the token section
-REVOCATION_CACHE_EXPIRATION_TIME = lambda: CONF.token.revocation_cache_time
+MEMOIZE = cache.get_memoization_decorator(section='revoke')
 
 
 def revoked_before_cutoff_time():
@@ -199,10 +197,9 @@ class Manager(manager.Manager):
     def revoke_by_domain_role_assignment(self, domain_id, role_id):
         self.revoke(model.RevokeEvent(domain_id=domain_id, role_id=role_id))
 
-    @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
-                        expiration_time=REVOCATION_CACHE_EXPIRATION_TIME)
+    @MEMOIZE
     def _get_revoke_tree(self):
-        events = self.driver.get_events()
+        events = self.driver.list_events()
         revoke_tree = model.RevokeTree(revoke_events=events)
 
         return revoke_tree
@@ -230,7 +227,7 @@ class Driver(object):
     """Interface for recording and reporting revocation events."""
 
     @abc.abstractmethod
-    def get_events(self, last_fetch=None):
+    def list_events(self, last_fetch=None):
         """return the revocation events, as a list of objects
 
         :param last_fetch:   Time of last fetch.  Return all events newer.
