@@ -14,12 +14,14 @@
 
 import hashlib
 
+from oslo_serialization import jsonutils
+
 from keystone.common import controller
 from keystone.common import dependency
-from keystone.common import driver_hints
+from keystone.common import validation
+from keystone.credential import schema
 from keystone import exception
 from keystone.i18n import _
-from keystone.openstack.common import jsonutils
 
 
 @dependency.requires('credential_api')
@@ -59,6 +61,7 @@ class CredentialV3(controller.V3Controller):
             return super(CredentialV3, self)._assign_unique_id(ref)
 
     @controller.protected()
+    @validation.validated(schema.credential_create, 'credential')
     def create_credential(self, context, credential):
         trust_id = self._get_trust_id_for_request(context)
         ref = self._assign_unique_id(self._normalize_dict(credential),
@@ -78,15 +81,13 @@ class CredentialV3(controller.V3Controller):
         else:
             return ref
 
-    @controller.protected()
-    def list_credentials(self, context):
-        # NOTE(henry-nash): Since there are no filters for credentials, we
-        # shouldn't limit the output, hence we don't pass a hints list into
-        # the driver.
-        refs = self.credential_api.list_credentials()
+    @controller.filterprotected('user_id')
+    def list_credentials(self, context, filters):
+        hints = CredentialV3.build_driver_hints(context, filters)
+        refs = self.credential_api.list_credentials(hints)
         ret_refs = [self._blob_to_json(r) for r in refs]
         return CredentialV3.wrap_collection(context, ret_refs,
-                                            driver_hints.Hints())
+                                            hints=hints)
 
     @controller.protected()
     def get_credential(self, context, credential_id):
@@ -95,6 +96,7 @@ class CredentialV3(controller.V3Controller):
         return CredentialV3.wrap_member(context, ret_ref)
 
     @controller.protected()
+    @validation.validated(schema.credential_update, 'credential')
     def update_credential(self, context, credential_id, credential):
         self._require_matching_id(credential_id, credential)
 
