@@ -16,17 +16,17 @@
 
 import abc
 
+from oslo_config import cfg
+from oslo_log import log
 import six
 
 from keystone.common import dependency
+from keystone.common import driver_hints
 from keystone.common import manager
-from keystone import config
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
-from keystone.openstack.common import log
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 
 LOG = log.getLogger(__name__)
 
@@ -43,6 +43,10 @@ class Manager(manager.Manager):
     def __init__(self):
         super(Manager, self).__init__(CONF.credential.driver)
 
+    @manager.response_truncated
+    def list_credentials(self, hints=None):
+        return self.driver.list_credentials(hints or driver_hints.Hints())
+
 
 @six.add_metaclass(abc.ABCMeta)
 class Driver(object):
@@ -55,16 +59,31 @@ class Driver(object):
         :raises: keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
-    def list_credentials(self, **filters):
-        """List all credentials in the system applying filters.
+    def list_credentials(self, hints):
+        """List all credentials.
+
+        :param hints: contains the list of filters yet to be satisfied.
+                      Any filters satisfied here will be removed so that
+                      the caller will know if any filters remain.
 
         :returns: a list of credential_refs or an empty list.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
+
+    @abc.abstractmethod
+    def list_credentials_for_user(self, user_id):
+        """List credentials for a user.
+
+        :param user_id: ID of a user to filter credentials by.
+
+        :returns: a list of credential_refs or an empty list.
+
+        """
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def get_credential(self, credential_id):
@@ -74,7 +93,7 @@ class Driver(object):
         :raises: keystone.exception.CredentialNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def update_credential(self, credential_id, credential):
@@ -84,7 +103,7 @@ class Driver(object):
                  keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_credential(self, credential_id):
@@ -93,7 +112,7 @@ class Driver(object):
         :raises: keystone.exception.CredentialNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
     @abc.abstractmethod
     def delete_credentials_for_project(self, project_id):
@@ -117,5 +136,5 @@ class Driver(object):
                 try:
                     self.credential_api.delete_credential(cr['id'])
                 except exception.CredentialNotFound:
-                    LOG.debug(_('Deletion of credential is not required: %s'),
+                    LOG.debug('Deletion of credential is not required: %s',
                               cr['id'])
