@@ -12,19 +12,16 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_config import cfg
+
 from keystone.common import sql
 from keystone.common import utils
-from keystone import config
 from keystone import exception
 from keystone.i18n import _
 from keystone import identity
 
-# Import assignment sql to ensure that the models defined in there are
-# available for the reference from User and Group to Domain.id.
-from keystone.assignment.backends import sql as assignment_sql  # noqa
 
-
-CONF = config.CONF
+CONF = cfg.CONF
 
 
 class User(sql.ModelBase, sql.DictBase):
@@ -33,8 +30,7 @@ class User(sql.ModelBase, sql.DictBase):
                   'default_project_id']
     id = sql.Column(sql.String(64), primary_key=True)
     name = sql.Column(sql.String(255), nullable=False)
-    domain_id = sql.Column(sql.String(64), sql.ForeignKey('domain.id'),
-                           nullable=False)
+    domain_id = sql.Column(sql.String(64), nullable=False)
     password = sql.Column(sql.String(128))
     enabled = sql.Column(sql.Boolean)
     extra = sql.Column(sql.JsonBlob())
@@ -55,8 +51,7 @@ class Group(sql.ModelBase, sql.DictBase):
     attributes = ['id', 'name', 'domain_id', 'description']
     id = sql.Column(sql.String(64), primary_key=True)
     name = sql.Column(sql.String(64), nullable=False)
-    domain_id = sql.Column(sql.String(64), sql.ForeignKey('domain.id'),
-                           nullable=False)
+    domain_id = sql.Column(sql.String(64), nullable=False)
     description = sql.Column(sql.Text())
     extra = sql.Column(sql.JsonBlob())
     # Unique constraint across two columns to create the separation
@@ -91,9 +86,6 @@ class Identity(identity.Driver):
     def _check_password(self, password, user_ref):
         """Check the specified password against the data store.
 
-        This is modeled on ldap/core.py.  The idea is to make it easier to
-        subclass Identity so that you can still use it to store all the data,
-        but use some other means to check the password.
         Note that we'll pass in the entire user_ref in case the subclass
         needs things like user_ref.get('name')
         For further justification, please see the follow up suggestion at
@@ -282,6 +274,17 @@ class Identity(identity.Driver):
         session = sql.get_session()
         return self._get_group(session, group_id).to_dict()
 
+    def get_group_by_name(self, group_name, domain_id):
+        session = sql.get_session()
+        query = session.query(Group)
+        query = query.filter_by(name=group_name)
+        query = query.filter_by(domain_id=domain_id)
+        try:
+            group_ref = query.one()
+        except sql.NotFound:
+            raise exception.GroupNotFound(group_id=group_name)
+        return group_ref.to_dict()
+
     @sql.handle_conflicts(conflict_type='group')
     def update_group(self, group_id, group):
         session = sql.get_session()
