@@ -16,24 +16,26 @@
 
 import abc
 
+from oslo_config import cfg
+import six
+
+from keystone.common import dependency
 from keystone.common import manager
-import keystone.conf
 from keystone import exception
 
-CONF = keystone.conf.CONF
+CONF = cfg.CONF
 
 
+@dependency.provider('id_generator_api')
 class Manager(manager.Manager):
     """Default pivot point for the identifier generator backend."""
 
-    driver_namespace = 'keystone.identity.id_generator'
-    _provides_api = 'id_generator_api'
-
     def __init__(self):
-        super().__init__(CONF.identity_mapping.generator)
+        super(Manager, self).__init__(CONF.identity_mapping.generator)
 
 
-class IDGenerator(metaclass=abc.ABCMeta):
+@six.add_metaclass(abc.ABCMeta)
+class IDGenerator(object):
     """Interface description for an ID Generator provider."""
 
     @abc.abstractmethod
