@@ -13,51 +13,61 @@
 
 import functools
 import os
+import shutil
 
 import fixtures
+from oslo_config import cfg
 from oslo_db import options as db_options
-from oslo_db.sqlalchemy import enginefacade
+from oslo_db.sqlalchemy import migration
 
 from keystone.common import sql
-import keystone.conf
-from keystone.tests import unit
+from keystone.common.sql import migration_helpers
+from keystone.tests import unit as tests
 
-CONF = keystone.conf.CONF
+
+CONF = cfg.CONF
 
 
 def run_once(f):
     """A decorator to ensure the decorated function is only executed once.
 
-    The decorated function is assumed to have a one parameter.
-
+    The decorated function cannot expect any arguments.
     """
-
     @functools.wraps(f)
     def wrapper():
         if not wrapper.already_ran:
             f()
             wrapper.already_ran = True
-
     wrapper.already_ran = False
     return wrapper
 
 
+def _setup_database(extensions=None):
+    if CONF.database.connection != tests.IN_MEM_DB_CONN_STRING:
+        db = tests.dirs.tmp('test.db')
+        pristine = tests.dirs.tmp('test.db.pristine')
+
+        if os.path.exists(db):
+            os.unlink(db)
+        if not os.path.exists(pristine):
+            migration.db_sync(sql.get_engine(),
+                              migration_helpers.find_migrate_repo())
+            for extension in (extensions or []):
+                migration_helpers.sync_database_to_version(extension=extension)
+            shutil.copyfile(db, pristine)
+        else:
+            shutil.copyfile(pristine, db)
+
+
 # NOTE(I159): Every execution all the options will be cleared. The method must
 # be called at the every fixture initialization.
-def initialize_sql_session(
-    connection_str=unit.IN_MEM_DB_CONN_STRING, enforce_sqlite_fks=True
-):
+def initialize_sql_session():
     # Make sure the DB is located in the correct location, in this case set
     # the default value, as this should be able to be overridden in some
     # test cases.
-    db_options.set_defaults(CONF, connection=connection_str)
-
-    # Enable the  Sqlite FKs for global engine by default.
-    facade = enginefacade.writer
-    engine = facade.get_engine()
-    f_key = 'ON' if enforce_sqlite_fks else 'OFF'
-    if engine.name == 'sqlite':
-        engine.connect().exec_driver_sql('PRAGMA foreign_keys = ' + f_key)
+    db_options.set_defaults(
+        CONF,
+        connection=tests.IN_MEM_DB_CONN_STRING)
 
 
 @run_once
@@ -77,48 +87,38 @@ def _load_sqlalchemy_models():
     the test run avoids this problem.
 
     """
-    keystone_root = os.path.normpath(
-        os.path.join(os.path.dirname(__file__), '..', '..', '..')
-    )
+    keystone_root = os.path.normpath(os.path.join(
+        os.path.dirname(__file__), '..', '..', '..'))
     for root, dirs, files in os.walk(keystone_root):
         # NOTE(morganfainberg): Slice the keystone_root off the root to ensure
         # we do not end up with a module name like:
         # Users.home.openstack.keystone.assignment.backends.sql
-        root = root[len(keystone_root) :]
+        root = root[len(keystone_root):]
         if root.endswith('backends') and 'sql.py' in files:
             # The root will be prefixed with an instance of os.sep, which will
             # make the root after replacement '.<root>', the 'keystone' part
             # of the module path is always added to the front
-            module_root = 'keystone.{}'.format(
-                root.replace(os.sep, '.').lstrip('.')
-            )
-            module_components = module_root.split('.')
-            module_without_backends = ''
-            for x in range(0, len(module_components) - 1):
-                module_without_backends += module_components[x] + '.'
-            module_without_backends = module_without_backends.rstrip('.')
-
-            module_name = module_root + '.sql'
+            module_name = ('keystone.%s.sql' %
+                           root.replace(os.sep, '.').lstrip('.'))
             __import__(module_name)
 
 
 class Database(fixtures.Fixture):
-    """A fixture for setting up and tearing down a database."""
+    """A fixture for setting up and tearing down a database.
 
-    def __init__(self):
-        super().__init__()
+    """
+
+    def __init__(self, extensions=None):
+        super(Database, self).__init__()
+        self._extensions = extensions
         initialize_sql_session()
         _load_sqlalchemy_models()
-        sql.enable_sqlite_foreign_key()
 
     def setUp(self):
-        super().setUp()
+        super(Database, self).setUp()
+        _setup_database(extensions=self._extensions)
 
-        with sql.session_for_write() as session:
-            self.engine = session.get_bind()
+        self.engine = sql.get_engine()
+        sql.ModelBase.metadata.create_all(bind=self.engine)
         self.addCleanup(sql.cleanup)
-        sql.ModelBase.metadata.create_all(bind=self.engine)
         self.addCleanup(sql.ModelBase.metadata.drop_all, bind=self.engine)
-
-    def recreate(self):
-        sql.ModelBase.metadata.create_all(bind=self.engine)
