@@ -14,23 +14,20 @@
 
 import os
 
-from six.moves import http_client
 import webtest
 
-from keystone.tests import unit
-from keystone.tests.unit.ksfixtures import database
+from keystone.tests import unit as tests
 
 
-class TestNoAdminTokenAuth(unit.TestCase):
+class TestNoAdminTokenAuth(tests.TestCase):
     def setUp(self):
         super(TestNoAdminTokenAuth, self).setUp()
-        self.useFixture(database.Database())
         self.load_backends()
 
         self._generate_paste_config()
 
         self.admin_app = webtest.TestApp(
-            self.loadapp(unit.dirs.tmp('no_admin_token_auth'), name='admin'),
+            self.loadapp(tests.dirs.tmp('no_admin_token_auth'), name='admin'),
             extra_environ=dict(REMOTE_ADDR='127.0.0.1'))
         self.addCleanup(setattr, self, 'admin_app', None)
 
@@ -38,12 +35,12 @@ class TestNoAdminTokenAuth(unit.TestCase):
         # Generate a file, based on keystone-paste.ini, that doesn't include
         # admin_token_auth in the pipeline
 
-        with open(unit.dirs.etc('keystone-paste.ini'), 'r') as f:
+        with open(tests.dirs.etc('keystone-paste.ini'), 'r') as f:
             contents = f.read()
 
         new_contents = contents.replace(' admin_token_auth ', ' ')
 
-        filename = unit.dirs.tmp('no_admin_token_auth-paste.ini')
+        filename = tests.dirs.tmp('no_admin_token_auth-paste.ini')
         with open(filename, 'w') as f:
             f.write(new_contents)
         self.addCleanup(os.remove, filename)
@@ -59,4 +56,4 @@ class TestNoAdminTokenAuth(unit.TestCase):
 
         # If the following does not raise, then the test is successful.
         self.admin_app.get(REQ_PATH, headers={'X-Auth-Token': 'NotAdminToken'},
-                           status=http_client.UNAUTHORIZED)
+                           status=401)
