@@ -12,7 +12,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.tests import unit
+from keystone.tests import unit as tests
+from keystone.tests.unit import test_sql_migrate_extensions
 from keystone.tests.unit import test_sql_upgrade
 
 
@@ -23,7 +24,7 @@ class PostgresqlMigrateTests(test_sql_upgrade.SqlUpgradeTests):
 
     def config_files(self):
         files = super(PostgresqlMigrateTests, self).config_files()
-        files.append(unit.dirs.tests_conf("backend_postgresql.conf"))
+        files.append(tests.dirs.tests_conf("backend_postgresql.conf"))
         return files
 
 
@@ -34,7 +35,30 @@ class MysqlMigrateTests(test_sql_upgrade.SqlUpgradeTests):
 
     def config_files(self):
         files = super(MysqlMigrateTests, self).config_files()
-        files.append(unit.dirs.tests_conf("backend_mysql.conf"))
+        files.append(tests.dirs.tests_conf("backend_mysql.conf"))
+        return files
+
+
+class PostgresqlRevokeExtensionsTests(
+        test_sql_migrate_extensions.RevokeExtension):
+    def setUp(self):
+        self.skip_if_env_not_set('ENABLE_LIVE_POSTGRES_TEST')
+        super(PostgresqlRevokeExtensionsTests, self).setUp()
+
+    def config_files(self):
+        files = super(PostgresqlRevokeExtensionsTests, self).config_files()
+        files.append(tests.dirs.tests_conf("backend_postgresql.conf"))
+        return files
+
+
+class MysqlRevokeExtensionsTests(test_sql_migrate_extensions.RevokeExtension):
+    def setUp(self):
+        self.skip_if_env_not_set('ENABLE_LIVE_MYSQL_TEST')
+        super(MysqlRevokeExtensionsTests, self).setUp()
+
+    def config_files(self):
+        files = super(MysqlRevokeExtensionsTests, self).config_files()
+        files.append(tests.dirs.tests_conf("backend_mysql.conf"))
         return files
 
 
@@ -45,5 +69,5 @@ class Db2MigrateTests(test_sql_upgrade.SqlUpgradeTests):
 
     def config_files(self):
         files = super(Db2MigrateTests, self).config_files()
-        files.append(unit.dirs.tests_conf("backend_db2.conf"))
+        files.append(tests.dirs.tests_conf("backend_db2.conf"))
         return files
