@@ -1,5 +1,3 @@
-# encoding: utf-8
-#
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,44 +13,45 @@
 # under the License.
 
 import gettext
-import os
+import random
+import socket
 import uuid
 
+import eventlet
 import mock
 import oslo_i18n
 from oslo_serialization import jsonutils
 import six
-from six.moves import http_client
 from testtools import matchers
 import webob
 
+from keystone.common import environment
 from keystone.common import wsgi
 from keystone import exception
-from keystone.server.flask import core as server_flask
-from keystone.tests import unit
+from keystone.tests import unit as tests
 
 
 class FakeApp(wsgi.Application):
-    def index(self, request):
+    def index(self, context):
         return {'a': 'b'}
 
 
 class FakeAttributeCheckerApp(wsgi.Application):
-    def index(self, request):
-        return request.params.mixed()
+    def index(self, context):
+        return context['query_string']
 
     def assert_attribute(self, body, attr):
-        """Assert that the given request has a certain attribute."""
+        """Asserts that the given request has a certain attribute."""
         ref = jsonutils.loads(body)
         self._require_attribute(ref, attr)
 
     def assert_attributes(self, body, attr):
-        """Assert that the given request has a certain set attributes."""
+        """Asserts that the given request has a certain set attributes."""
         ref = jsonutils.loads(body)
         self._require_attributes(ref, attr)
 
 
-class RouterTest(unit.TestCase):
+class RouterTest(tests.TestCase):
     def setUp(self):
         self.router = wsgi.RoutersBase()
         super(RouterTest, self).setUp()
@@ -68,7 +67,7 @@ class RouterTest(unit.TestCase):
                           status=uuid.uuid4().hex)
 
 
-class BaseWSGITest(unit.TestCase):
+class BaseWSGITest(tests.TestCase):
     def setUp(self):
         self.app = FakeApp()
         super(BaseWSGITest, self).setUp()
@@ -84,15 +83,26 @@ class ApplicationTest(BaseWSGITest):
     def test_response_content_type(self):
         req = self._make_request()
         resp = req.get_response(self.app)
-        self.assertEqual('application/json', resp.content_type)
+        self.assertEqual(resp.content_type, 'application/json')
 
     def test_query_string_available(self):
         class FakeApp(wsgi.Application):
-            def index(self, request):
-                return request.params.mixed()
+            def index(self, context):
+                return context['query_string']
         req = self._make_request(url='/?1=2')
         resp = req.get_response(FakeApp())
-        self.assertEqual({'1': '2'}, jsonutils.loads(resp.body))
+        self.assertEqual(jsonutils.loads(resp.body), {'1': '2'})
+
+    def test_headers_available(self):
+        class FakeApp(wsgi.Application):
+            def index(self, context):
+                return context['headers']
+
+        app = FakeApp()
+        req = self._make_request(url='/?1=2')
+        req.headers['X-Foo'] = "bar"
+        resp = req.get_response(app)
+        self.assertIn('X-Foo', eval(resp.body))
 
     def test_render_response(self):
         data = {'attribute': 'value'}
@@ -100,17 +110,15 @@ class ApplicationTest(BaseWSGITest):
 
         resp = wsgi.render_response(body=data)
         self.assertEqual('200 OK', resp.status)
-        self.assertEqual(http_client.OK, resp.status_int)
+        self.assertEqual(200, resp.status_int)
         self.assertEqual(body, resp.body)
         self.assertEqual('X-Auth-Token', resp.headers.get('Vary'))
         self.assertEqual(str(len(body)), resp.headers.get('Content-Length'))
 
     def test_render_response_custom_status(self):
-        resp = wsgi.render_response(
-            status=(http_client.NOT_IMPLEMENTED,
-                    http_client.responses[http_client.NOT_IMPLEMENTED]))
+        resp = wsgi.render_response(status=(501, 'Not Implemented'))
         self.assertEqual('501 Not Implemented', resp.status)
-        self.assertEqual(http_client.NOT_IMPLEMENTED, resp.status_int)
+        self.assertEqual(501, resp.status_int)
 
     def test_successful_require_attribute(self):
         app = FakeAttributeCheckerApp()
@@ -159,29 +167,19 @@ class ApplicationTest(BaseWSGITest):
         self.assertEqual('Some-Value', resp.headers.get('Custom-Header'))
         self.assertEqual('X-Auth-Token', resp.headers.get('Vary'))
 
-    def test_render_response_non_str_headers_converted(self):
-        resp = wsgi.render_response(
-            headers=[('Byte-Header', 'Byte-Value'),
-                     (u'Unicode-Header', u'Unicode-Value')])
-        # assert that all headers are identified.
-        self.assertEqual('Unicode-Value', resp.headers.get('Unicode-Header'))
-        # assert that unicode value is converted, the expected type is str
-        # on both python2 and python3.
-        self.assertEqual(str,
-                         type(resp.headers.get('Unicode-Header')))
-
     def test_render_response_no_body(self):
         resp = wsgi.render_response()
         self.assertEqual('204 No Content', resp.status)
-        self.assertEqual(http_client.NO_CONTENT, resp.status_int)
+        self.assertEqual(204, resp.status_int)
         self.assertEqual(b'', resp.body)
+        self.assertEqual('0', resp.headers.get('Content-Length'))
         self.assertIsNone(resp.headers.get('Content-Type'))
 
     def test_render_response_head_with_body(self):
         resp = wsgi.render_response({'id': uuid.uuid4().hex}, method='HEAD')
-        self.assertEqual(http_client.OK, resp.status_int)
+        self.assertEqual(200, resp.status_int)
         self.assertEqual(b'', resp.body)
-        self.assertNotEqual('0', resp.headers.get('Content-Length'))
+        self.assertNotEqual(resp.headers.get('Content-Length'), '0')
         self.assertEqual('application/json', resp.headers.get('Content-Type'))
 
     def test_application_local_config(self):
@@ -196,215 +194,14 @@ class ApplicationTest(BaseWSGITest):
     def test_render_exception(self):
         e = exception.Unauthorized(message=u'\u7f51\u7edc')
         resp = wsgi.render_exception(e)
-        self.assertEqual(http_client.UNAUTHORIZED, resp.status_int)
+        self.assertEqual(401, resp.status_int)
 
     def test_render_exception_host(self):
         e = exception.Unauthorized(message=u'\u7f51\u7edc')
-        req = self._make_request(url='/')
-        context = {'host_url': 'http://%s:5000' % uuid.uuid4().hex,
-                   'environment': req.environ}
+        context = {'host_url': 'http://%s:5000' % uuid.uuid4().hex}
         resp = wsgi.render_exception(e, context=context)
 
-        self.assertEqual(http_client.UNAUTHORIZED, resp.status_int)
-
-    def test_improperly_encoded_params(self):
-        class FakeApp(wsgi.Application):
-            def index(self, request):
-                return request.params.mixed()
-        # this is high bit set ASCII, copy & pasted from Windows.
-        # aka code page 1252. It is not valid UTF8.
-        req = self._make_request(url='/?name=nonexit%E8nt')
-        self.assertRaises(exception.ValidationError, req.get_response,
-                          FakeApp())
-
-    def test_properly_encoded_params(self):
-        class FakeApp(wsgi.Application):
-            def index(self, request):
-                return request.params.mixed()
-        # nonexitènt encoded as UTF-8
-        req = self._make_request(url='/?name=nonexit%C3%A8nt')
-        resp = req.get_response(FakeApp())
-        self.assertEqual({'name': u'nonexit\xe8nt'},
-                         jsonutils.loads(resp.body))
-
-    def test_base_url(self):
-        class FakeApp(wsgi.Application):
-            def index(self, request):
-                return self.base_url(request.context_dict)
-        req = self._make_request(url='/')
-        # NOTE(gyee): according to wsgiref, if HTTP_HOST is present in the
-        # request environment, it will be used to construct the base url.
-        # SERVER_NAME and SERVER_PORT will be ignored. These are standard
-        # WSGI environment variables populated by the webserver.
-        req.environ.update({
-            'SCRIPT_NAME': '/identity',
-            'SERVER_NAME': '1.2.3.4',
-            'wsgi.url_scheme': 'http',
-            'SERVER_PORT': '80',
-            'HTTP_HOST': '1.2.3.4',
-        })
-        resp = req.get_response(FakeApp())
-        self.assertEqual(b"http://1.2.3.4/identity", resp.body)
-
-        # if HTTP_HOST is absent, SERVER_NAME and SERVER_PORT will be used
-        req = self._make_request(url='/')
-        del req.environ['HTTP_HOST']
-        req.environ.update({
-            'SCRIPT_NAME': '/identity',
-            'SERVER_NAME': '1.1.1.1',
-            'wsgi.url_scheme': 'http',
-            'SERVER_PORT': '1234',
-        })
-        resp = req.get_response(FakeApp())
-        self.assertEqual(b"http://1.1.1.1:1234/identity", resp.body)
-
-        # make sure keystone normalize the standard HTTP port 80 by stripping
-        # it
-        req = self._make_request(url='/')
-        req.environ.update({'HTTP_HOST': 'foo:80',
-                            'SCRIPT_NAME': '/identity'})
-        resp = req.get_response(FakeApp())
-        self.assertEqual(b"http://foo/identity", resp.body)
-
-        # make sure keystone normalize the standard HTTPS port 443 by stripping
-        # it
-        req = self._make_request(url='/')
-        req.environ.update({'HTTP_HOST': 'foo:443',
-                            'SCRIPT_NAME': '/identity',
-                            'wsgi.url_scheme': 'https'})
-        resp = req.get_response(FakeApp())
-        self.assertEqual(b"https://foo/identity", resp.body)
-
-        # make sure non-standard port is preserved
-        req = self._make_request(url='/')
-        req.environ.update({'HTTP_HOST': 'foo:1234',
-                            'SCRIPT_NAME': '/identity'})
-        resp = req.get_response(FakeApp())
-        self.assertEqual(b"http://foo:1234/identity", resp.body)
-
-        # make sure version portion of the SCRIPT_NAME, '/v3' is stripped from
-        # base url
-        req = self._make_request(url='/')
-        req.environ.update({'HTTP_HOST': 'foo:80',
-                            'SCRIPT_NAME': '/identity/v3'})
-        resp = req.get_response(FakeApp())
-        self.assertEqual(b"http://foo/identity", resp.body)
-
-
-class WSGIAppConfigTest(unit.TestCase):
-    default_config_file = 'keystone.conf'
-    custom_config_dir = '/etc/kst/'
-    custom_config_files = ['kst.conf', 'kst2.conf']
-
-    def test_config_files_have_default_values_when_envars_not_set(self):
-        config_files = server_flask._get_config_files()
-        config_files.sort()
-        expected_config_files = []
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_config_files_have_default_values_with_empty_envars(self):
-        env = {'OS_KEYSTONE_CONFIG_FILES': '',
-               'OS_KEYSTONE_CONFIG_DIR': ''}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = []
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_can_use_single_config_file_under_default_config_dir(self):
-        cfg = self.custom_config_files[0]
-        env = {'OS_KEYSTONE_CONFIG_FILES': cfg}
-        config_files = server_flask._get_config_files(env)
-        expected_config_files = [cfg]
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_can_use_multiple_config_files_under_default_config_dir(self):
-        env = {'OS_KEYSTONE_CONFIG_FILES': ';'.join(self.custom_config_files)}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = self.custom_config_files
-        self.assertListEqual(config_files, expected_config_files)
-
-        config_with_empty_strings = self.custom_config_files + ['', ' ']
-        env = {'OS_KEYSTONE_CONFIG_FILES': ';'.join(config_with_empty_strings)}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_can_use_single_absolute_path_config_file(self):
-        cfg = self.custom_config_files[0]
-        cfgpath = os.path.join(self.custom_config_dir, cfg)
-        env = {'OS_KEYSTONE_CONFIG_FILES': cfgpath}
-        config_files = server_flask._get_config_files(env)
-        self.assertListEqual(config_files, [cfgpath])
-
-    def test_can_use_multiple_absolute_path_config_files(self):
-        cfgpaths = [os.path.join(self.custom_config_dir, cfg)
-                    for cfg in self.custom_config_files]
-        cfgpaths.sort()
-        env = {'OS_KEYSTONE_CONFIG_FILES': ';'.join(cfgpaths)}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        self.assertListEqual(config_files, cfgpaths)
-
-        env = {'OS_KEYSTONE_CONFIG_FILES': ';'.join(cfgpaths + ['', ' '])}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        self.assertListEqual(config_files, cfgpaths)
-
-    def test_can_use_default_config_files_with_custom_config_dir(self):
-        env = {'OS_KEYSTONE_CONFIG_DIR': self.custom_config_dir}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = [os.path.join(self.custom_config_dir,
-                                              self.default_config_file)]
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_can_use_single_config_file_under_custom_config_dir(self):
-        cfg = self.custom_config_files[0]
-        env = {'OS_KEYSTONE_CONFIG_DIR': self.custom_config_dir,
-               'OS_KEYSTONE_CONFIG_FILES': cfg}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = [os.path.join(self.custom_config_dir, cfg)]
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_can_use_multiple_config_files_under_custom_config_dir(self):
-        env = {'OS_KEYSTONE_CONFIG_DIR': self.custom_config_dir,
-               'OS_KEYSTONE_CONFIG_FILES': ';'.join(self.custom_config_files)}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = [os.path.join(self.custom_config_dir, s)
-                                 for s in self.custom_config_files]
-        expected_config_files.sort()
-        self.assertListEqual(config_files, expected_config_files)
-
-        config_with_empty_strings = self.custom_config_files + ['', ' ']
-        env = {'OS_KEYSTONE_CONFIG_DIR': self.custom_config_dir,
-               'OS_KEYSTONE_CONFIG_FILES': ';'.join(config_with_empty_strings)}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        self.assertListEqual(config_files, expected_config_files)
-
-    def test_can_mix_relative_and_absolute_paths_config_file(self):
-        cfg0 = self.custom_config_files[0]
-        cfgpath0 = os.path.join(self.custom_config_dir,
-                                self.custom_config_files[0])
-        cfgpath1 = os.path.join(self.custom_config_dir,
-                                self.custom_config_files[1])
-        env = {'OS_KEYSTONE_CONFIG_DIR': self.custom_config_dir,
-               'OS_KEYSTONE_CONFIG_FILES': ';'.join([cfg0, cfgpath1])}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = [cfgpath0, cfgpath1]
-        expected_config_files.sort()
-        self.assertListEqual(config_files, expected_config_files)
-
-        env = {'OS_KEYSTONE_CONFIG_FILES': ';'.join([cfg0, cfgpath1])}
-        config_files = server_flask._get_config_files(env)
-        config_files.sort()
-        expected_config_files = [cfg0, cfgpath1]
-        expected_config_files.sort()
-        self.assertListEqual(config_files, expected_config_files)
+        self.assertEqual(401, resp.status_int)
 
 
 class ExtensionRouterTest(BaseWSGITest):
@@ -418,23 +215,6 @@ class ExtensionRouterTest(BaseWSGITest):
         self.assertIn("testkey", app.kwargs)
         self.assertEqual("test", app.kwargs["testkey"])
 
-    def test_resource_not_found_message(self):
-        class FakeRouter(wsgi.ExtensionRouter):
-            def __init__(self, *args, **kwargs):
-                self.kwargs = kwargs
-
-        factory = FakeRouter.factory({}, testkey="test")
-        app = factory(self.app)
-        req = webob.Request.blank('/WHATWHA')
-        # Force the match in the Router to fail so we can verify
-        # that the URL is included in the 404 error message.
-        req.environ['wsgiorg.routing_args'] = [None, None]
-        resp = app._dispatch(req)
-        body = jsonutils.loads(resp.body)
-        self.assertEqual(body['error']['message'],
-                         u'(http://localhost/WHATWHA): The resource could '
-                         'not be found.')
-
 
 class MiddlewareTest(BaseWSGITest):
     def test_middleware_request(self):
@@ -491,17 +271,26 @@ class MiddlewareTest(BaseWSGITest):
             self.assertEqual(exception.UnexpectedError.code, resp.status_int)
             return resp
 
-        # Exception data should not be in the message when insecure_debug is
-        # False
-        self.config_fixture.config(debug=False, insecure_debug=False)
+        # Exception data should not be in the message when debug is False
+        self.config_fixture.config(debug=False)
         self.assertNotIn(exception_str, do_request().body)
 
-        # Exception data should be in the message when insecure_debug is True
-        self.config_fixture.config(debug=True, insecure_debug=True)
+        # Exception data should be in the message when debug is True
+        self.config_fixture.config(debug=True)
         self.assertIn(exception_str, do_request().body)
 
+    def test_middleware_local_config(self):
+        class FakeMiddleware(wsgi.Middleware):
+            def __init__(self, *args, **kwargs):
+                self.kwargs = kwargs
 
-class LocalizedResponseTest(unit.TestCase):
+        factory = FakeMiddleware.factory({}, testkey="test")
+        app = factory(self.app)
+        self.assertIn("testkey", app.kwargs)
+        self.assertEqual("test", app.kwargs["testkey"])
+
+
+class LocalizedResponseTest(tests.TestCase):
     def test_request_match_default(self):
         # The default language if no Accept-Language is provided is None
         req = webob.Request.blank('/')
@@ -512,7 +301,7 @@ class LocalizedResponseTest(unit.TestCase):
         # If Accept-Language is a supported language, best_match_language()
         # returns it.
 
-        language = 'bogus'
+        language = uuid.uuid4().hex
         mock_gal.return_value = [language]
 
         req = webob.Request.blank('/', headers={'Accept-Language': language})
@@ -534,8 +323,8 @@ class LocalizedResponseTest(unit.TestCase):
     def test_static_translated_string_is_lazy_translatable(self):
         # Statically created message strings are an object that can get
         # lazy-translated rather than a regular string.
-        self.assertNotEqual(six.text_type,
-                            type(exception.Unauthorized.message_format))
+        self.assertNotEqual(type(exception.Unauthorized.message_format),
+                            six.text_type)
 
     @mock.patch.object(oslo_i18n, 'get_available_languages')
     def test_get_localized_response(self, mock_gal):
@@ -543,7 +332,7 @@ class LocalizedResponseTest(unit.TestCase):
         # and an exception is raised by the application that is translatable
         # then the response will have the translated message.
 
-        language = 'bogus'
+        language = uuid.uuid4().hex
         mock_gal.return_value = [language]
 
         # The arguments for the xlated message format have to match the args
@@ -579,7 +368,7 @@ class LocalizedResponseTest(unit.TestCase):
             # Fake app raises NotFound exception to simulate Keystone raising.
 
             class FakeApp(wsgi.Application):
-                def index(self, request):
+                def index(self, context):
                     raise exception.NotFound(target=target)
 
             # Make the request with Accept-Language on the app, expect an error
@@ -597,3 +386,102 @@ class LocalizedResponseTest(unit.TestCase):
             self.assertThat(resp.json['error']['message'],
                             matchers.Equals(exp_msg))
             self.assertThat(xlation_mock.called, matchers.Equals(True))
+
+
+class ServerTest(tests.TestCase):
+
+    def setUp(self):
+        super(ServerTest, self).setUp()
+        self.host = '127.0.0.1'
+        self.port = '1234'
+
+    @mock.patch('eventlet.listen')
+    @mock.patch('socket.getaddrinfo')
+    def test_keepalive_unset(self, mock_getaddrinfo, mock_listen):
+        mock_getaddrinfo.return_value = [(1, 2, 3, 4, 5)]
+        mock_sock_dup = mock_listen.return_value.dup.return_value
+
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port)
+        server.start()
+        self.addCleanup(server.stop)
+        self.assertTrue(mock_listen.called)
+        self.assertFalse(mock_sock_dup.setsockopt.called)
+
+    @mock.patch('eventlet.listen')
+    @mock.patch('socket.getaddrinfo')
+    def test_keepalive_set(self, mock_getaddrinfo, mock_listen):
+        mock_getaddrinfo.return_value = [(1, 2, 3, 4, 5)]
+        mock_sock_dup = mock_listen.return_value.dup.return_value
+
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port, keepalive=True)
+        server.start()
+        self.addCleanup(server.stop)
+        mock_sock_dup.setsockopt.assert_called_once_with(socket.SOL_SOCKET,
+                                                         socket.SO_KEEPALIVE,
+                                                         1)
+        self.assertTrue(mock_listen.called)
+
+    @mock.patch('eventlet.listen')
+    @mock.patch('socket.getaddrinfo')
+    def test_keepalive_and_keepidle_set(self, mock_getaddrinfo, mock_listen):
+        mock_getaddrinfo.return_value = [(1, 2, 3, 4, 5)]
+        mock_sock_dup = mock_listen.return_value.dup.return_value
+
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port, keepalive=True,
+                                    keepidle=1)
+        server.start()
+        self.addCleanup(server.stop)
+
+        self.assertEqual(2, mock_sock_dup.setsockopt.call_count)
+
+        # Test the last set of call args i.e. for the keepidle
+        mock_sock_dup.setsockopt.assert_called_with(socket.IPPROTO_TCP,
+                                                    socket.TCP_KEEPIDLE,
+                                                    1)
+
+        self.assertTrue(mock_listen.called)
+
+    def test_client_socket_timeout(self):
+        # mocking server method of eventlet.wsgi to check it is called with
+        # configured 'client_socket_timeout' value.
+
+        # setting random socket_timeout between 1 to 10, so that after given
+        # seconds client connection will be closed if it is idle.
+        socket_timeout = random.randint(1, 10)
+        self.config_fixture.config(group='eventlet_server',
+                                   client_socket_timeout=socket_timeout)
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port)
+        with mock.patch.object(eventlet.wsgi, 'server') as mock_server:
+            fake_application = uuid.uuid4().hex
+            fake_socket = uuid.uuid4().hex
+            server._run(fake_application, fake_socket)
+            mock_server.assert_called_once_with(fake_socket,
+                                                fake_application,
+                                                debug=mock.ANY,
+                                                socket_timeout=socket_timeout,
+                                                log=mock.ANY,
+                                                keepalive=mock.ANY)
+
+    def test_wsgi_keep_alive(self):
+        # mocking server method of eventlet.wsgi to check it is called with
+        # configured 'wsgi_keep_alive' value.
+        wsgi_keepalive = False
+        self.config_fixture.config(group='eventlet_server',
+                                   wsgi_keep_alive=wsgi_keepalive)
+
+        server = environment.Server(mock.MagicMock(), host=self.host,
+                                    port=self.port)
+        with mock.patch.object(eventlet.wsgi, 'server') as mock_server:
+            fake_application = uuid.uuid4().hex
+            fake_socket = uuid.uuid4().hex
+            server._run(fake_application, fake_socket)
+            mock_server.assert_called_once_with(fake_socket,
+                                                fake_application,
+                                                debug=mock.ANY,
+                                                socket_timeout=mock.ANY,
+                                                log=mock.ANY,
+                                                keepalive=wsgi_keepalive)
