@@ -12,27 +12,108 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from oslo.utils import timeutils
+from oslo_config import cfg
+from oslo_log import log
+from oslo_serialization import jsonutils
+from oslo_utils import timeutils
 import six
 from six.moves.urllib import parse
 
+from keystone.common import controller as common_controller
 from keystone.common import dependency
-from keystone import config
 from keystone.contrib import federation
 from keystone import exception
-from keystone.i18n import _
-from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log
+from keystone.i18n import _, _LE
+from keystone.openstack.common import versionutils
 from keystone import token
 from keystone.token import provider
 
 
 LOG = log.getLogger(__name__)
-CONF = config.CONF
+CONF = cfg.CONF
 
 
+@dependency.requires('catalog_api', 'resource_api')
 class V2TokenDataHelper(object):
     """Creates V2 token data."""
+
+    def v3_to_v2_token(self, token_id, v3_token_data):
+        token_data = {}
+        # Build v2 token
+        v3_token = v3_token_data['token']
+
+        token = {}
+        token['id'] = token_id
+        token['expires'] = v3_token.get('expires_at')
+        token['issued_at'] = v3_token.get('issued_at')
+        token['audit_ids'] = v3_token.get('audit_ids')
+
+        # Bail immediately if this is a domain-scoped token, which is not
+        # supported by the v2 API at all.
+        if 'domain' in v3_token:
+            raise exception.Unauthorized(_(
+                'Domains are not supported by the v2 API. Please use the v3 '
+                'API instead.'))
+
+        # Bail if this is a project-scoped token outside the default domain,
+        # which may result in a namespace collision with a project inside the
+        # default domain.
+        if 'project' in v3_token:
+            if (v3_token['project']['domain']['id'] !=
+                    CONF.identity.default_domain_id):
+                raise exception.Unauthorized(_(
+                    'Project not found in the default domain (please use the '
+                    'v3 API instead): %s') % v3_token['project']['id'])
+
+            # v3 token_data does not contain all tenant attributes
+            tenant = self.resource_api.get_project(
+                v3_token['project']['id'])
+            token['tenant'] = common_controller.V2Controller.filter_domain_id(
+                tenant)
+        token_data['token'] = token
+
+        # Build v2 user
+        v3_user = v3_token['user']
+
+        # Bail if this is a token outside the default domain,
+        # which may result in a namespace collision with a project inside the
+        # default domain.
+        if ('domain' in v3_user and v3_user['domain']['id'] !=
+                CONF.identity.default_domain_id):
+            raise exception.Unauthorized(_(
+                'User not found in the default domain (please use the v3 API '
+                'instead): %s') % v3_user['id'])
+
+        user = common_controller.V2Controller.v3_to_v2_user(v3_user)
+
+        # Set user roles
+        user['roles'] = []
+        role_ids = []
+        for role in v3_token.get('roles', []):
+            # Filter role id since it's not included in v2 token response
+            role_ids.append(role.pop('id'))
+            user['roles'].append(role)
+        user['roles_links'] = []
+
+        token_data['user'] = user
+
+        # Get and build v2 service catalog
+        token_data['serviceCatalog'] = []
+        if 'tenant' in token:
+            catalog_ref = self.catalog_api.get_catalog(
+                user['id'], token['tenant']['id'])
+            if catalog_ref:
+                token_data['serviceCatalog'] = self.format_catalog(catalog_ref)
+
+        # Build v2 metadata
+        metadata = {}
+        metadata['roles'] = role_ids
+        # Setting is_admin to keep consistency in v2 response
+        metadata['is_admin'] = 0
+        token_data['metadata'] = metadata
+
+        return {'access': token_data}
+
     @classmethod
     def format_token(cls, token_ref, roles_ref=None, catalog_ref=None,
                      trust_ref=None):
@@ -143,8 +224,8 @@ class V2TokenDataHelper(object):
         return services.values()
 
 
-@dependency.requires('assignment_api', 'catalog_api', 'identity_api',
-                     'trust_api')
+@dependency.requires('assignment_api', 'catalog_api', 'federation_api',
+                     'identity_api', 'resource_api', 'role_api', 'trust_api')
 class V3TokenDataHelper(object):
     """Token data helper."""
     def __init__(self):
@@ -152,11 +233,11 @@ class V3TokenDataHelper(object):
         super(V3TokenDataHelper, self).__init__()
 
     def _get_filtered_domain(self, domain_id):
-        domain_ref = self.assignment_api.get_domain(domain_id)
+        domain_ref = self.resource_api.get_domain(domain_id)
         return {'id': domain_ref['id'], 'name': domain_ref['name']}
 
     def _get_filtered_project(self, project_id):
-        project_ref = self.assignment_api.get_project(project_id)
+        project_ref = self.resource_api.get_project(project_id)
         filtered_project = {
             'id': project_ref['id'],
             'name': project_ref['name']}
@@ -182,7 +263,7 @@ class V3TokenDataHelper(object):
         if project_id:
             roles = self.assignment_api.get_roles_for_user_and_project(
                 user_id, project_id)
-        return [self.assignment_api.get_role(role_id) for role_id in roles]
+        return [self.role_api.get_role(role_id) for role_id in roles]
 
     def _populate_roles_for_groups(self, group_ids,
                                    project_id=None, domain_id=None,
@@ -256,7 +337,7 @@ class V3TokenDataHelper(object):
         if access_token:
             filtered_roles = []
             authed_role_ids = jsonutils.loads(access_token['role_ids'])
-            all_roles = self.assignment_api.list_roles()
+            all_roles = self.role_api.list_roles()
             for role in all_roles:
                 for authed_role in authed_role_ids:
                     if authed_role == role['id']:
@@ -325,6 +406,14 @@ class V3TokenDataHelper(object):
             # TODO(ayoung): Enforce Endpoints for trust
             token_data['catalog'] = service_catalog
 
+    def _populate_service_providers(self, token_data):
+        if 'service_providers' in token_data:
+            return
+
+        service_providers = self.federation_api.get_enabled_service_providers()
+        if service_providers:
+            token_data['service_providers'] = service_providers
+
     def _populate_token_dates(self, token_data, expires=None, trust=None,
                               issued_at=None):
         if not expires:
@@ -341,16 +430,23 @@ class V3TokenDataHelper(object):
         elif isinstance(audit_info, list):
             token_data['audit_ids'] = audit_info
         else:
-            msg = _('Invalid audit info data type: %(data)s (%(type)s)')
-            msg_subst = {'data': audit_info, 'type': type(audit_info)}
-            LOG.error(msg, msg_subst)
-            raise exception.UnexpectedError(msg % msg_subst)
+            msg = (_('Invalid audit info data type: %(data)s (%(type)s)') %
+                   {'data': audit_info, 'type': type(audit_info)})
+            LOG.error(msg)
+            raise exception.UnexpectedError(msg)
 
-    def get_token_data(self, user_id, method_names, extras,
+    def get_token_data(self, user_id, method_names, extras=None,
                        domain_id=None, project_id=None, expires=None,
                        trust=None, token=None, include_catalog=True,
                        bind=None, access_token=None, issued_at=None,
                        audit_info=None):
+        if extras is None:
+            extras = {}
+        if extras:
+            versionutils.deprecated(
+                what='passing token data with "extras"',
+                as_of=versionutils.deprecated.KILO,
+                in_favor_of='well-defined APIs')
         token_data = {'methods': method_names,
                       'extras': extras}
 
@@ -376,15 +472,15 @@ class V3TokenDataHelper(object):
         if include_catalog:
             self._populate_service_catalog(token_data, user_id, domain_id,
                                            project_id, trust)
+        self._populate_service_providers(token_data)
         self._populate_token_dates(token_data, expires=expires, trust=trust,
                                    issued_at=issued_at)
         self._populate_oauth_section(token_data, access_token)
         return {'token': token_data}
 
 
-@dependency.optional('oauth_api')
-@dependency.requires('assignment_api', 'catalog_api', 'identity_api',
-                     'trust_api')
+@dependency.requires('catalog_api', 'identity_api', 'oauth_api',
+                     'resource_api', 'role_api', 'trust_api')
 class BaseProvider(provider.Provider):
     def __init__(self, *args, **kwargs):
         super(BaseProvider, self).__init__(*args, **kwargs)
@@ -419,6 +515,10 @@ class BaseProvider(provider.Provider):
         token_data['access']['token']['id'] = token_id
         return token_id, token_data
 
+    def _is_mapped_token(self, auth_context):
+        return (federation.IDENTITY_PROVIDER in auth_context and
+                federation.PROTOCOL in auth_context)
+
     def issue_v3_token(self, user_id, method_names, expires_at=None,
                        project_id=None, domain_id=None, auth_context=None,
                        trust=None, metadata_ref=None, include_catalog=True,
@@ -429,17 +529,14 @@ class BaseProvider(provider.Provider):
             trust = self.trust_api.get_trust(metadata_ref['trust_id'])
 
         token_ref = None
-        if 'saml2' in method_names:
-            token_ref = self._handle_saml2_tokens(auth_context, project_id,
-                                                  domain_id)
+        if auth_context and self._is_mapped_token(auth_context):
+            token_ref = self._handle_mapped_tokens(
+                auth_context, project_id, domain_id)
 
         access_token = None
         if 'oauth1' in method_names:
-            if self.oauth_api:
-                access_token_id = auth_context['access_token_id']
-                access_token = self.oauth_api.get_access_token(access_token_id)
-            else:
-                raise exception.Forbidden(_('Oauth is disabled.'))
+            access_token_id = auth_context['access_token_id']
+            access_token = self.oauth_api.get_access_token(access_token_id)
 
         token_data = self.v3_token_data_helper.get_token_data(
             user_id,
@@ -458,7 +555,12 @@ class BaseProvider(provider.Provider):
         token_id = self._get_token_id(token_data)
         return token_id, token_data
 
-    def _handle_saml2_tokens(self, auth_context, project_id, domain_id):
+    def _handle_mapped_tokens(self, auth_context, project_id, domain_id):
+        def get_federated_domain():
+            return (CONF.federation.federated_domain_name or
+                    federation.FEDERATED_DOMAIN_KEYWORD)
+
+        federated_domain = get_federated_domain()
         user_id = auth_context['user_id']
         group_ids = auth_context['group_ids']
         idp = auth_context[federation.IDENTITY_PROVIDER]
@@ -470,6 +572,10 @@ class BaseProvider(provider.Provider):
                 federation.FEDERATION: {
                     'identity_provider': {'id': idp},
                     'protocol': {'id': protocol}
+                },
+                'domain': {
+                    'id': federated_domain,
+                    'name': federated_domain
                 }
             }
         }
@@ -490,6 +596,16 @@ class BaseProvider(provider.Provider):
             raise exception.Unauthorized()
         return token_ref
 
+    def _assert_is_not_federation_token(self, token_ref):
+        """Make sure we aren't using v2 auth on a federation token."""
+        token_data = token_ref.get('token_data')
+        if (token_data and self.get_token_version(token_data) ==
+                token.provider.V3):
+            if 'OS-FEDERATION' in token_data['token']['user']:
+                msg = _('Attempting to use OS-FEDERATION token with V2 '
+                        'Identity Service, use V3 Authentication')
+                raise exception.Unauthorized(msg)
+
     def _assert_default_domain(self, token_ref):
         """Make sure we are operating on default domain only."""
         if (token_ref.get('token_data') and
@@ -528,7 +644,7 @@ class BaseProvider(provider.Provider):
                 if (trustor_user_ref['domain_id'] !=
                         CONF.identity.default_domain_id):
                     raise exception.Unauthorized(msg)
-                project_ref = self.assignment_api.get_project(
+                project_ref = self.resource_api.get_project(
                     trust_ref['project_id'])
                 if (project_ref['domain_id'] !=
                         CONF.identity.default_domain_id):
@@ -536,6 +652,7 @@ class BaseProvider(provider.Provider):
 
     def validate_v2_token(self, token_ref):
         try:
+            self._assert_is_not_federation_token(token_ref)
             self._assert_default_domain(token_ref)
             # FIXME(gyee): performance or correctness? Should we return the
             # cached token or reconstruct it? Obviously if we are going with
@@ -553,7 +670,7 @@ class BaseProvider(provider.Provider):
                 metadata_ref = token_ref['metadata']
                 roles_ref = []
                 for role_id in metadata_ref.get('roles', []):
-                    roles_ref.append(self.assignment_api.get_role(role_id))
+                    roles_ref.append(self.role_api.get_role(role_id))
 
                 # Get a service catalog if possible
                 # This is needed for on-behalf-of requests
@@ -561,8 +678,7 @@ class BaseProvider(provider.Provider):
                 if token_ref.get('tenant'):
                     catalog_ref = self.catalog_api.get_catalog(
                         token_ref['user']['id'],
-                        token_ref['tenant']['id'],
-                        metadata_ref)
+                        token_ref['tenant']['id'])
 
                 trust_ref = None
                 if CONF.trust.enabled and 'trust_id' in metadata_ref:
@@ -571,9 +687,15 @@ class BaseProvider(provider.Provider):
 
                 token_data = self.v2_token_data_helper.format_token(
                     token_ref, roles_ref, catalog_ref, trust_ref)
+
+            trust_id = token_data['access'].get('trust', {}).get('id')
+            if trust_id:
+                # token trust validation
+                self.trust_api.get_trust(trust_id)
+
             return token_data
         except exception.ValidationError as e:
-            LOG.exception(_('Failed to validate token'))
+            LOG.exception(_LE('Failed to validate token'))
             raise exception.TokenNotFound(e)
 
     def validate_v3_token(self, token_ref):
@@ -585,6 +707,12 @@ class BaseProvider(provider.Provider):
         # Lets go with the cached token strategy. Since token
         # management layer is now pluggable, one can always provide
         # their own implementation to suit their needs.
+
+        trust_id = token_ref.get('trust_id')
+        if trust_id:
+            # token trust validation
+            self.trust_api.get_trust(trust_id)
+
         token_data = token_ref.get('token_data')
         if not token_data or 'token' not in token_data:
             # token ref is created by V2 API
@@ -599,7 +727,6 @@ class BaseProvider(provider.Provider):
             token_data = self.v3_token_data_helper.get_token_data(
                 token_ref['user']['id'],
                 ['password', 'token'],
-                {},
                 project_id=project_id,
                 bind=token_ref.get('bind'),
                 expires=token_ref['expires'],
