@@ -25,33 +25,29 @@ LOG = log.getLogger(__name__)
 CONF = cfg.CONF
 
 
-def validate_key_repository(requires_write=False):
+def validate_key_repository():
     """Validate permissions on the key repository directory."""
     # NOTE(lbragstad): We shouldn't need to check if the directory was passed
     # in as None because we don't set allow_no_values to True.
 
-    # ensure current user has sufficient access to the key repository
-    is_valid = (os.access(CONF.fernet_tokens.key_repository, os.R_OK) and
-                os.access(CONF.fernet_tokens.key_repository, os.X_OK))
-    if requires_write:
-        is_valid = (is_valid and
-                    os.access(CONF.fernet_tokens.key_repository, os.W_OK))
-
-    if not is_valid:
+    # ensure current user has full access to the key repository
+    if (not os.access(CONF.fernet_tokens.key_repository, os.R_OK) or not
+            os.access(CONF.fernet_tokens.key_repository, os.W_OK) or not
+            os.access(CONF.fernet_tokens.key_repository, os.X_OK)):
         LOG.error(
             _LE('Either [fernet_tokens] key_repository does not exist or '
                 'Keystone does not have sufficient permission to access it: '
                 '%s'), CONF.fernet_tokens.key_repository)
-    else:
-        # ensure the key repository isn't world-readable
-        stat_info = os.stat(CONF.fernet_tokens.key_repository)
-        if(stat_info.st_mode & stat.S_IROTH or
-           stat_info.st_mode & stat.S_IXOTH):
-            LOG.warning(_LW(
-                '[fernet_tokens] key_repository is world readable: %s'),
-                CONF.fernet_tokens.key_repository)
+        return False
 
-    return is_valid
+    # ensure the key repository isn't world-readable
+    stat_info = os.stat(CONF.fernet_tokens.key_repository)
+    if stat_info.st_mode & stat.S_IROTH or stat_info.st_mode & stat.S_IXOTH:
+        LOG.warning(_LW(
+            '[fernet_tokens] key_repository is world readable: %s'),
+            CONF.fernet_tokens.key_repository)
+
+    return True
 
 
 def _convert_to_integers(id_value):
@@ -63,8 +59,8 @@ def _convert_to_integers(id_value):
     try:
         id_int = int(id_value)
     except ValueError as e:
-        msg = _LE('Unable to convert Keystone user or group ID. Error: %s')
-        LOG.error(msg, e)
+        msg = ('Unable to convert Keystone user or group ID. Error: %s', e)
+        LOG.error(msg)
         raise
 
     return id_int
@@ -103,7 +99,7 @@ def _create_new_key(keystone_user_id, keystone_group_id):
 
     Create a new key that is readable by the Keystone group and Keystone user.
     """
-    key = fernet.Fernet.generate_key()  # key is bytes
+    key = fernet.Fernet.generate_key()
 
     # This ensures the key created is not world-readable
     old_umask = os.umask(0o177)
@@ -121,7 +117,7 @@ def _create_new_key(keystone_user_id, keystone_group_id):
     key_file = os.path.join(CONF.fernet_tokens.key_repository, '0')
     try:
         with open(key_file, 'w') as f:
-            f.write(key.decode('utf-8'))  # convert key to str for the file.
+            f.write(key)
     finally:
         # After writing the key, set the umask back to it's original value. Do
         # the same with group and user identifiers if a Keystone group or user
@@ -180,14 +176,14 @@ def rotate_keys(keystone_user_id=None, keystone_group_id=None):
         if os.path.isfile(path):
             try:
                 key_id = int(filename)
-            except ValueError:  # nosec : name isn't a number, ignore the file.
+            except ValueError:
                 pass
             else:
                 key_files[key_id] = path
 
     LOG.info(_LI('Starting key rotation with %(count)s key files: %(list)s'), {
         'count': len(key_files),
-        'list': list(key_files.values())})
+        'list': key_files.values()})
 
     # determine the number of the new primary key
     current_primary_key = max(key_files.keys())
@@ -208,24 +204,22 @@ def rotate_keys(keystone_user_id=None, keystone_group_id=None):
     # add a new key to the rotation, which will be the *next* primary
     _create_new_key(keystone_user_id, keystone_group_id)
 
-    max_active_keys = CONF.fernet_tokens.max_active_keys
     # check for bad configuration
-    if max_active_keys < 1:
+    if CONF.fernet_tokens.max_active_keys < 1:
         LOG.warning(_LW(
             '[fernet_tokens] max_active_keys must be at least 1 to maintain a '
             'primary key.'))
-        max_active_keys = 1
+        CONF.fernet_tokens.max_active_keys = 1
 
     # purge excess keys
-
-    # Note that key_files doesn't contain the new active key that was created,
-    # only the old active keys.
-    keys = sorted(key_files.keys(), reverse=True)
-    while len(keys) > (max_active_keys - 1):
-        index_to_purge = keys.pop()
-        key_to_purge = key_files[index_to_purge]
-        LOG.info(_LI('Excess key to purge: %s'), key_to_purge)
-        os.remove(key_to_purge)
+    keys = sorted(key_files.keys())
+    number_of_keys_to_purge = max(
+        0, len(key_files) - CONF.fernet_tokens.max_active_keys + 1)
+    if number_of_keys_to_purge > 0:
+        excess_keys = keys[:number_of_keys_to_purge]
+        LOG.info(_LI('Excess keys to purge: %s'), excess_keys)
+        for i in excess_keys:
+            os.remove(key_files[i])
 
 
 def load_keys():
@@ -247,8 +241,7 @@ def load_keys():
             with open(path, 'r') as key_file:
                 try:
                     key_id = int(filename)
-                except ValueError:  # nosec : filename isn't a number, ignore
-                    # this file since it's not a key.
+                except ValueError:
                     pass
                 else:
                     keys[key_id] = key_file.read()
@@ -260,10 +253,8 @@ def load_keys():
         # the number of keys matches max_active_keys, this log entry is too
         # repetitive to be useful.
         LOG.info(_LI(
-            'Loaded %(count)d encryption keys (max_active_keys=%(max)d) from: '
-            '%(dir)s'), {
+            'Loaded %(count)s encryption keys from: %(dir)s'), {
                 'count': len(keys),
-                'max': CONF.fernet_tokens.max_active_keys,
                 'dir': CONF.fernet_tokens.key_repository})
 
     # return the encryption_keys, sorted by key number, descending
