@@ -15,17 +15,18 @@
 """Keystone PKI Token Provider"""
 
 from keystoneclient.common import cms
+from oslo_config import cfg
+from oslo_log import log
+from oslo_serialization import jsonutils
 
 from keystone.common import environment
-from keystone import config
+from keystone.common import utils
 from keystone import exception
-from keystone.i18n import _
-from keystone.openstack.common import jsonutils
-from keystone.openstack.common import log
+from keystone.i18n import _, _LE
 from keystone.token.providers import common
 
 
-CONF = config.CONF
+CONF = cfg.CONF
 
 LOG = log.getLogger(__name__)
 
@@ -37,11 +38,16 @@ class Provider(common.BaseProvider):
             # produces unicode.  This can be removed if the client returns
             # str()
             # TODO(ayoung): Make to a byte_str for Python3
-            token_id = str(cms.cms_sign_token(jsonutils.dumps(token_data),
+            token_json = jsonutils.dumps(token_data, cls=utils.PKIEncoder)
+            token_id = str(cms.cms_sign_token(token_json,
                                               CONF.signing.certfile,
                                               CONF.signing.keyfile))
             return token_id
         except environment.subprocess.CalledProcessError:
-            LOG.exception(_('Unable to sign token'))
+            LOG.exception(_LE('Unable to sign token'))
             raise exception.UnexpectedError(_(
                 'Unable to sign token.'))
+
+    def needs_persistence(self):
+        """Should the token be written to a backend."""
+        return True
