@@ -14,11 +14,11 @@
 
 import time
 
-from oslo.utils import timeutils
+from oslo_log import log
+from oslo_utils import timeutils
 
 from keystone.common import sql
 from keystone import exception
-from keystone.openstack.common import log
 from keystone import trust
 
 
@@ -117,7 +117,7 @@ class Trust(trust.Driver):
                     raise exception.TrustUseLimitReached(trust_id=trust_id)
             # NOTE(morganfainberg): Ensure we have a yield point for eventlet
             # here. This should cost us nothing otherwise. This can be removed
-            # if/when oslo.db cleanly handles yields on db calls.
+            # if/when oslo_db cleanly handles yields on db calls.
             time.sleep(0)
         else:
             # NOTE(morganfainberg): In the case the for loop is not prematurely
